package jp.savethememory.manage.action;

import java.io.IOException;
import java.io.InputStream;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.ResourceBundle;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipOutputStream;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import jp.savethememory.manage.dto.DispSyashinDto;
import jp.savethememory.manage.dto.UserDto;
import jp.savethememory.manage.entity.TbKoukaiSyashin;
import jp.savethememory.manage.entity.TbShinsei;
import jp.savethememory.manage.entity.TbSyashin;
import jp.savethememory.manage.form.TbShinseiForm;
import jp.savethememory.manage.form.TbSyashinForm;
import jp.savethememory.manage.service.StorageS3Service;
import jp.savethememory.manage.service.TbKoukaiSyashinService;
import jp.savethememory.manage.service.TbShichousonService;
import jp.savethememory.manage.service.TbShinseiService;
import jp.savethememory.manage.service.TbSyashinService;
import jp.savethememory.manage.util.CommonDefine;
import jp.savethememory.manage.util.LoginChecker;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.lf5.util.StreamUtils;
import org.seasar.extension.jdbc.exception.SOptimisticLockException;
import org.seasar.framework.beans.util.Beans;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.ResponseUtil;

/**
 * TbShinseiAction 申請情報の表示と保存を行います。 原本返却に伴い、必要な処理を行います。 原本返却処理完了時には完了画面の表示を行います。
 *
 *
 */
public class TbShinseiAction {

	private static final String DOWNLOAD_ALL_PICTURES_FILE_NAME = "pictures.zip";
	@ActionForm
	@Resource
	public TbShinseiForm tbShinseiForm;

	@ActionForm
	@Resource
	protected TbSyashinForm tbSyashinForm;

	public UserDto userDto;

	public List<TbShinsei> tbShinseiItems;
	public List<TbSyashin> tbSyashinShinseiNoItems;
	public List<TbKoukaiSyashin> tbKoukaiSyashinShinseiNoItems;
	public List<DispSyashinDto> dispSyashinItems;
	// public List<TbSyashin> albumSyashinItems;
	public int syashinCount;
	public int koukaiSyashinCount;

	@Resource
	protected TbShinseiService tbShinseiService;

	@Resource
	protected TbSyashinService tbSyashinService;

	@Resource
	protected TbKoukaiSyashinService tbKoukaiSyashinService;

	@Resource
	protected TbShichousonService tbShichousonService;

	private LoginChecker loginChecker = new LoginChecker();

	protected Log LOGGER = LogFactory.getLog(TbShinseiAction.class);

	/** 　排他制御時のメッセージ */
	private static final String EXCLUSIVE_CONTROL_ERROR = "他のユーザーにより更新済みです。内容をご確認の上再度操作をお願いいたします。";
	/** 　何かエラーが発生した場合のメッセージ */
	private static final String SYSTEM_ERROR = "エラーが発生しました。再度操作をするか、管理者に連絡してください。";
	private static Pattern ptn = Pattern.compile("^[0-9]{2}[01][0-9][0-3][0-9]");
	private static int FILE_NAME_SUBSTRING_PREFIX_LENGTH = 7;

	/**
	 * 編集画面の表示
	 *
	 * @return
	 */
	@Execute(validator = false, urlPattern = "edit/{shinseiNo}")
	public String edit() {
		if (!loginChecker.isLogin(userDto)) {
			return "/login?redirect=true";
		}
		TbShinsei entity = tbShinseiService.findById(Long.valueOf(tbShinseiForm.shinseiNo));
		Beans.copy(entity, tbShinseiForm).dateConverter("yyyy/MM/dd", "birthday", "shinseiDay").execute();
		if (tbShinseiForm.mailAddress != null) {
			tbShinseiForm.mailAddress = tbShinseiForm.mailAddress.trim();
		}
		this.tbShinseiForm.inputMailAddress = "";
		this.tbShinseiForm.retypeMailAddress = "";

		// 写真一覧（原本あり）
		this.tbSyashinShinseiNoItems = tbSyashinService.findByShinseiNo(tbShinseiForm.shinseiNo);
		setDispSyashinItems();

		// 写真一覧（原本なし）
		this.tbKoukaiSyashinShinseiNoItems = tbKoukaiSyashinService.findByShinseiNo(tbShinseiForm.shinseiNo);
		for (TbKoukaiSyashin syashin : tbKoukaiSyashinShinseiNoItems) {
			// ファイル名の先頭が日付の場合は削除して表示
			if (ptn.matcher(syashin.getFileName()).find()) {
				syashin.setFileName(syashin.getFileName().substring(FILE_NAME_SUBSTRING_PREFIX_LENGTH));
			}
		}
		this.koukaiSyashinCount = this.tbKoukaiSyashinShinseiNoItems.size();

		return "edit.jsp";

	}

	/**
	 * 写真一覧（原本あり）の表示項目を設定
	 */
	private void setDispSyashinItems() {

		this.dispSyashinItems = new ArrayList<DispSyashinDto>();
		this.syashinCount = 0;

		for (TbSyashin syashin : this.tbSyashinShinseiNoItems) {
			String fileName = syashin.getFileName().trim();
			DispSyashinDto dispSyashin = new DispSyashinDto();
			if (CommonDefine.FILE_NAME_PATTERN1.matcher(fileName).find()) {
				// ファイル名の先頭が日付の場合は削除して表示
				dispSyashin.title = fileName.substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH);
				dispSyashin.titlePrefix = dispSyashin.title.substring(0, dispSyashin.title.length() - CommonDefine.FILE_NAME_SUFFIX_LENGTH);
			} else if (CommonDefine.FILE_NAME_PATTERN2.matcher(fileName).find()) {
				// ファイル名の先頭が日付かつ８枚ものの場合
				dispSyashin.title = fileName.substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH);
				dispSyashin.titlePrefix = dispSyashin.title.substring(0, dispSyashin.title.length() - CommonDefine.FILE_NAME_EIGHT_SUFFIX_LENGTH);
			} else if (CommonDefine.FILE_NAME_PATTERN3.matcher(fileName).find()) {
				// ファイル名の先頭が日付ではない場合
				dispSyashin.title = fileName;
				dispSyashin.titlePrefix = dispSyashin.title.substring(0, dispSyashin.title.length() - CommonDefine.FILE_NAME_SUFFIX_LENGTH);
			}

			boolean contains = false;
			if (syashin.getQuanpUploadStatus() == null) {
				syashin.setQuanpUploadStatus(0);
			}
			for (DispSyashinDto dSyashin : this.dispSyashinItems) {
				if (dSyashin.titlePrefix.equals(dispSyashin.titlePrefix)) {
					// 同じアルバムの写真のquanpアップロードステータスの最低値を採用する
					if (dSyashin.quanpUploadStatus > syashin.getQuanpUploadStatus()) {
						dSyashin.quanpUploadStatus = syashin.getQuanpUploadStatus();
					}
					if (syashin.getExcludeFlg() != null && syashin.getExcludeFlg()) {
						dSyashin.excludeContentsCount++;
					}
					dSyashin.albumContentsCount++;
					this.syashinCount++;
					contains = true;
					break;
				}
			}

			if (!contains) {
				this.syashinCount++;
				dispSyashin.albumContentsCount = 1;
				dispSyashin.id = syashin.getContentsId().toString();
				dispSyashin.quanpUploadStatus = syashin.getQuanpUploadStatus();
				dispSyashin.returnFlg = syashin.getReturnFlg();
				if (syashin.getExcludeFlg() != null && syashin.getExcludeFlg()) {
					dispSyashin.excludeContentsCount = 1;
				} else {
					dispSyashin.excludeContentsCount = 0;
				}
				this.dispSyashinItems.add(dispSyashin);

			}
		}
	}

	/**
	 * 申請テーブルの更新可能項目 <br/>
	 * ・スタッフコメント（いつでも変更可） <br/>
	 * ・申請ステータス（申請中から（原本）引渡済への変更のみ可）<br/>
	 * ・デジタル返却ステータス（申請ステータスが、（原本）引渡済の場合のみ、未返却から返却済に変更可）<br/>
	 *
	 * 1.申請ステータスが、申請中の場合 スタッフコメントの編集のみと考えられるため、申請テーブルのみ更新する。 画面遷移は行わない<br/>
	 * 2.申請ステータスが、引渡済で、申請テーブルのステータスも引き渡し済みにしていた場合<br/>
	 * 既に原本引渡処理済みなので、threadによるプレイス移動等は行わない<br/>
	 * ただし、デジタルステータスが引渡済になっている場合は、写真テーブルのquanpUploadStatusをアップロード完了に更新する。<br/>
	 * 画面遷移は行わない 3.申請ステータスが、引渡済で、申請テーブルのステータスが申請中の場合 原本返却処理を行う。<br/>
	 * メアドがない場合は
	 * 、申請テーブルのquanpIdStatusを発行不要とし、写真テーブルのquanpUploadStatusをアップロード不要にする。<br/>
	 * メアドがある場合は、 quanpIDの発行を行う。（リピーターの場合は以前発行したものを使う）<br/>
	 * quanpIDが取得できた場合は、写真テーブルのquanpUploadStatusをアップロード要にする。<br/>
	 * quanpIDが取得できなかった場合は、写真テーブルのquanpUploadStatusをアップロード不要にする。<br/>
	 * メアドのある、無しにかかわらず写真テーブルのコンテンツは全てthreadによるプレイス移動等を行う 完了画面を表示する。<br/>
	 *
	 * @return returnPage 画面遷移するjsp
	 * @throws Exception
	 */
	@Execute(validator = false)
	public String update() throws Exception {
		LOGGER.debug("start-----.");
		if (!loginChecker.isLogin(userDto)) {
			return "/login?redirect=true";
		}

		tbShinseiForm.message = null;
		String returnPage = null;
		TbShinsei beforeShinsei = null;
		beforeShinsei = tbShinseiService.findById(Long.parseLong(tbShinseiForm.shinseiNo));
		TbShinsei tbShinsei = null;
		tbShinsei = Beans.createAndCopy(TbShinsei.class, tbShinseiForm).dateConverter("yyyy/MM/dd").execute();

		try {
			LOGGER.debug("shinseiNo:" + tbShinseiForm.shinseiNo + ".");
			LOGGER.debug("shinseiStatus:" + tbShinseiForm.shinseiStatus + ".");
			LOGGER.debug("digitalStatus:" + tbShinseiForm.digitalStatus + ".");
			LOGGER.debug("mailAddress:" + tbShinseiForm.mailAddress + ".");
			LOGGER.debug("quanpIdStatus:" + tbShinseiForm.quanpIdStatus + ".");

			if (CommonDefine.SHINSEI_STATUS_SHINSEICHU.equals(tbShinseiForm.shinseiStatus)) {
				/** 未返却の場合(申請中→申請中) **/
				returnPage = "/tbShinsei/edit/" + tbShinseiForm.shinseiNo + "?redirect=true";
			} else if (CommonDefine.SHINSEI_STATUS_HIKIWATASHIZUMI.equals(tbShinseiForm.shinseiStatus) && CommonDefine.SHINSEI_STATUS_HIKIWATASHIZUMI.equals(beforeShinsei.getShinseiStatus().toString())) {
				/** 以前引き渡し済みにしていた場合（引き渡し済み→引き渡し済み） **/
				if (CommonDefine.DIGITAL_STATUS_RETURN.equals(tbShinseiForm.digitalStatus)) {
					// デジタルステータスが"返却済み"の場合
					// TODO
					// 複数回動作する既に更新したい状態になっていたら、更新しないようにupdateQuanpUploadStatusを変更する？
					// 20130213 AWS移行時 同じ値を更新しているため、コメントアウト
					// tbSyashinService.updateQuanpUploadStatus(
					// tbShinseiForm.shinseiNo,
					// CommonDefine.QUANP_UPLOAD_STATUS_SUCCESS);
				}
				returnPage = "/tbShinsei/edit/" + tbShinseiForm.shinseiNo + "?redirect=true";
			} else if (CommonDefine.SHINSEI_STATUS_HIKIWATASHIZUMI.equals(tbShinseiForm.shinseiStatus) && CommonDefine.SHINSEI_STATUS_SHINSEICHU.equals(beforeShinsei.getShinseiStatus().toString())) {
				/** 原本を初めて引き渡す場合、原本引渡処理を開始する **/
				// 除外写真を申請対象から外す
				tbSyashinService.updateToExcludeFlgIsTrue(tbShinseiForm.shinseiNo);
				this.tbShinseiForm.returnCount = tbSyashinService.updateRetuneFlgtoTrue(tbShinseiForm.shinseiNo);
				// 申請番号を写真のサマリーに追加
				tbSyashinService.setSummary(tbShinseiForm.shinseiNo);

				// 公開写真も引き渡した枚数としてカウント
				List<TbKoukaiSyashin> list = tbKoukaiSyashinService.findByShinseiNo(tbShinseiForm.shinseiNo);
				if (!list.isEmpty()) {
					this.tbShinseiForm.returnCount += list.size();
				}

				//市町村テーブルのリターンカウントを更新
				tbShichousonService.updateReturnCount(userDto.account);

				returnPage = "/tbShinsei/updated?redirect=true";
			}

			tbShinsei.setUpdateDay(new Timestamp((new Date()).getTime()));
			tbShinseiService.update(tbShinsei);
			LOGGER.debug("TbShinsei updated.");

			// 更新した内容（entity）を一旦formに読み込み直す。次画面で表示時に使うため。
			Beans.copy(tbShinsei, tbShinseiForm).dateConverter("yyyy/MM/dd", "birthday", "shinseiDay").execute();

		} catch (SOptimisticLockException e) {
			// 楽観的排他制御が発生した場合は、画面上に表示します。
			tbShinseiForm.message = EXCLUSIVE_CONTROL_ERROR;
			e.printStackTrace();
			LOGGER.error(e.getMessage(), e);
			returnPage = "/tbShinsei/edit/" + tbShinseiForm.shinseiNo + "?redirect=true";
		} catch (Exception e) {
			// エラーが発生していることは画面上にだします。
			tbShinseiForm.message = SYSTEM_ERROR;
			e.printStackTrace();
			LOGGER.error(e.getMessage(), e);
			returnPage = "/tbShinsei/edit/" + tbShinseiForm.shinseiNo + "?redirect=true";
		}

		LOGGER.debug("end-----.");
		return returnPage;
	}

	/**
	 * 完了画面を表示します。
	 *
	 * @return
	 */
	@Execute(validator = false)
	public String updated() {
		LOGGER.debug("start-----.");
		if (!loginChecker.isLogin(userDto)) {
			LOGGER.debug("end-----.");
			return "/login?redirect=true";
		}
		LOGGER.debug("end-----.");
		return "complete.jsp";
	}

	/**
	 * 申請一覧画面へリダイレクトします
	 *
	 * @return
	 */
	@Execute(validator = false)
	public String redirectList() {
		tbShinseiForm.message = null;
		return "/tbShinseiSearch/list?redirect=true";
	}

	/**
	 * アルバム写真一覧画面を表示します
	 */
	@Execute(validator = false, urlPattern = "albumSyashinList/{contentId}&{canExclude}")
	public String albumSyashinList() {
		TbSyashin syashin = tbSyashinService.findByContentId(this.tbShinseiForm.contentId);
		List<String> list = new ArrayList<String>();
		this.tbShinseiForm.albumSyashinItems = new ArrayList<TbSyashin>();

		if (syashin != null) {
			String title = "";
			String titlePrefix = "";

			if (CommonDefine.FILE_NAME_PATTERN1.matcher(syashin.getFileName()).find()) {
				// ファイル名の先頭が日付の場合は削除して表示
				title = syashin.getFileName().substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH);
				titlePrefix = title.substring(0, title.length() - CommonDefine.FILE_NAME_SUFFIX_LENGTH);
			} else if (CommonDefine.FILE_NAME_PATTERN2.matcher(syashin.getFileName()).find()) {
				// ファイル名の先頭が日付かつ８枚ものの場合
				title = syashin.getFileName().substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH);
				titlePrefix = title.substring(0, title.length() - CommonDefine.FILE_NAME_EIGHT_SUFFIX_LENGTH);
			} else if (CommonDefine.FILE_NAME_PATTERN3.matcher(syashin.getFileName()).find()) {
				title = syashin.getFileName();
				titlePrefix = title.substring(0, title.length() - CommonDefine.FILE_NAME_SUFFIX_LENGTH);
			} else {
				title = syashin.getFileName();
				titlePrefix = title;
			}

			this.tbShinseiForm.albumSyashinItems = this.tbSyashinService.findByShinseiNoAndFileName(syashin.getShinseiNo().toString(), titlePrefix);

			if (this.tbShinseiForm.albumSyashinItems != null) {
				for (TbSyashin s : this.tbShinseiForm.albumSyashinItems) {
					if (s.getExcludeFlg() != null && s.getExcludeFlg()) {
						list.add(s.getContentsId().toString());
					}
					if (CommonDefine.FILE_NAME_PATTERN1.matcher(s.getFileName()).find()) {
						// ファイル名の先頭が日付の場合は削除して表示
						s.setFileName(s.getFileName().substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH));
					} else if (CommonDefine.FILE_NAME_PATTERN2.matcher(syashin.getFileName()).find()) {
						// ファイル名の先頭が日付かつ８枚ものの場合
						s.setFileName(s.getFileName().substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH));
					}
				}
			}
			this.tbShinseiForm.multibox = (String[]) list.toArray(new String[0]);
		}
		return "albumSyashinList.jsp";
	}

	/**
	 * 写真一覧画面で除外写真を確定し、申請情報画面に戻ります
	 */
	@Execute(validator = false)
	public String defineExclude() {
		this.tbSyashinService.updateExcludeFlg(this.tbShinseiForm.albumSyashinItems, this.tbShinseiForm.multibox);
		return edit();
	}

	/**
	 * デジタル写真を返却するメソッド
	 * s3からコンテンツを取得し、responseでzipファイルを返します。また、tb_syashinテーブルのuploadstatusを3
	 * 、tb_shinseiテーブルのdigitalStatusを１に更新します。
	 *
	 * @return edit()編集画面へ遷移するメソッド
	 * @throws IOException
	 */
	@Execute(validator = false, urlPattern = "returnDigitalData/{shinseiNo}")
	public String returnDigitalData() throws IOException {
		if (!loginChecker.isLogin(userDto)) {
			return "/login?redirect=true";
		}

		HttpServletResponse response = ResponseUtil.getResponse();
		response.setContentType("application/zip; charset=UTF-8");
		response.setHeader("Content-Disposition", "attachment;filename=" + DOWNLOAD_ALL_PICTURES_FILE_NAME);

		StorageS3Service s3 = new StorageS3Service();
		ZipOutputStream zos = new ZipOutputStream(response.getOutputStream());

		ResourceBundle bundle = null;
		bundle = ResourceBundle.getBundle(CommonDefine.PROPERTIES_STMM_FILE_NAME);
		try {
			// 原本ありの写真を取得
			this.tbSyashinShinseiNoItems = tbSyashinService.findByShinseiNo(tbShinseiForm.shinseiNo);

			for (TbSyashin syashin : tbSyashinShinseiNoItems) {
				// ファイルキーの作成
				String key = bundle.getString(CommonDefine.PROPERTIES_STMM_KEY_IMG_ORIGINAL_PREFIX) + syashin.getContentsId() + bundle.getString(CommonDefine.PROPERTIES_STMM_KEY_IMG_SUFFIX);
				InputStream inFile = null;
				try {
					String fileName = syashin.getFileName();
					inFile = s3.downloadFile(userDto.account, key);
					writeFileToZip(zos, fileName, inFile);
				} finally {
					IOUtils.closeQuietly(inFile);
				}
			}

			// 原本なしの写真を取得
			this.tbKoukaiSyashinShinseiNoItems = tbKoukaiSyashinService.findByShinseiNo(tbShinseiForm.shinseiNo);

			for (TbKoukaiSyashin koukaiSyashin : tbKoukaiSyashinShinseiNoItems) {
				// ファイルキーの作成
				String key = bundle.getString(CommonDefine.PROPERTIES_STMM_KEY_IMG_ORIGINAL_PREFIX) + koukaiSyashin.getContentsId() + bundle.getString(CommonDefine.PROPERTIES_STMM_KEY_IMG_SUFFIX);
				InputStream inFile = null;
				try {
					String fileName = koukaiSyashin.getFileName();
					inFile = s3.downloadFile(userDto.account, key);
					writeFileToZip(zos, fileName, inFile);
				} finally {
					inFile.close();
				}
			}

			// 申請テーブルのデジタルステータスを更新
			tbShinseiService.updateDigitalStatus(tbShinseiForm.shinseiNo, Integer.parseInt(CommonDefine.DIGITAL_STATUS_RETURN));

			// 写真テーブルのアップロードステータスを更新
			tbSyashinService.updateQuanpUploadStatus(tbShinseiForm.shinseiNo, CommonDefine.QUANP_UPLOAD_STATUS_DOWNLOADED);

			// 公開写真テーブルのアップロードステータスを更新
			tbKoukaiSyashinService.updateQuanpUploadStatus(tbShinseiForm.shinseiNo, CommonDefine.QUANP_UPLOAD_STATUS_DOWNLOADED);

		} catch (ZipException e) {
			response.setStatus(HttpServletResponse.SC_BAD_GATEWAY);
			LOGGER.error(e);
		} catch (IOException e) {
			response.setStatus(HttpServletResponse.SC_BAD_GATEWAY);
			LOGGER.error(e);
		} catch (Throwable e) {
			response.setStatus(HttpServletResponse.SC_BAD_GATEWAY);//BAD GATEWAY を指定すると、IEはダウンロード失敗ダイアログを出してくれる。
			LOGGER.error(e);
		} finally {
			try{
			if (zos != null) {
				zos.closeEntry();
				zos.close();
			}
			}catch (ZipException e){
				response.setStatus(HttpServletResponse.SC_BAD_GATEWAY);
				LOGGER.error(e);
			}
		}

		return null;

	}

	private void writeFileToZip(ZipOutputStream zos, String fileName, InputStream inFile) throws IOException {
		ZipEntry ze = new ZipEntry(fileName);
		zos.putNextEntry(ze);
		StreamUtils.copy(inFile, zos);
		zos.closeEntry();
	}
}