package jp.savethememory.manage.action;



import java.util.List;
import java.util.ResourceBundle;

import javax.annotation.Resource;

import jp.savethememory.manage.dto.UserDto;
import jp.savethememory.manage.entity.TbShinsei;
import jp.savethememory.manage.form.TbShinseiForm;
import jp.savethememory.manage.form.TbShinseiSearchForm;
import jp.savethememory.manage.service.TbShinseiService;
import jp.savethememory.manage.util.CommonDefine;
import jp.savethememory.manage.util.LoginChecker;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.seasar.framework.beans.util.Beans;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;

/**
 * TbShinseiSearchAction
 * 申請情報の検索画面を表示します。
 * 全て条件はアンド検索になります。
 * 検索実行後に申請情報一覧を表示染ます
 *
 */
public class TbShinseiSearchAction {

	public UserDto userDto;
	public List<TbShinsei> tbShinseiItems;
	public long totalPage;
	public long total;

    @ActionForm
    @Resource
    protected TbShinseiSearchForm tbShinseiSearchForm;

    protected TbShinseiForm tbShinseiForm;
    @Resource
    protected TbShinseiService tbShinseiService;

    protected Log LOGGER = LogFactory.getLog(TbShinseiSearchAction.class);

    private LoginChecker loginChecker = new LoginChecker();
    private ResourceBundle PROPERTIES = ResourceBundle.getBundle(CommonDefine.PROPERTIES_STMM_FILE_NAME);


    /**
     * 申請検索画面を表示します。
     * @return
     */
    @Execute(validator = false)
    public String index() {
    	LOGGER.debug("start-----.");

    	if(!loginChecker.isLogin(userDto)){
    		return "/login?redirect=true";
    	}

    	if(tbShinseiSearchForm.shichousonName == null){
    		tbShinseiSearchForm.shichousonName = userDto.shichousonName;
    		tbShinseiSearchForm.initialize();
    	}
    	LOGGER.debug("end-----.");
    	return "index.jsp";
    }


    /**
     * ページを初期値にした上で申請一覧を表示します。
     * @return
     */
    @Execute(validator = true, input = "index.jsp")
    public String search() {

    	tbShinseiSearchForm.page = 1;
    	return list();
    }

    /**
     * 申請一覧を表示します。
     * @return
     */
    @Execute(validator = true, input = "index.jsp",urlPattern = "list")
    public String list() {
    	LOGGER.debug("start-----.");
    	if(!loginChecker.isLogin(userDto)){
    		return "/login?redirect=true";
    	}

    	//件数を取得
    	total = tbShinseiService.getTbShinseiItemsCount(tbShinseiSearchForm);
    	totalPage = 0;

    	long max = Long.parseLong(PROPERTIES.getString(CommonDefine.PROPERTIES_STMM_MAX_NUMBER_OF_SHINSEI_IN_A_PAGE));

    	tbShinseiItems = null;
    	if(total > 0){
    		while(true){
    			if((total % max) == 0){
    				totalPage = total / max;
    			} else {
    				totalPage = total / max + 1;
    			}
	    		tbShinseiItems = tbShinseiService.getTbShinseiItems(tbShinseiSearchForm);
	        	if(!tbShinseiItems.isEmpty()){
	        		Beans.copy(tbShinseiItems, tbShinseiItems).dateConverter("yyyy/MM/dd","birthday","shinseiDay").execute();
	        		break;
	        	} else {
	        		//ページ内に表示する情報が存在しない場合は、ページを1減らす
	        		tbShinseiSearchForm.page--;
	        	}
    		}
    	}

    	LOGGER.debug("end-----.");
    	return "list.jsp";
    }



    /**
     * ユーザーが入力した、申請条件を一旦リセットします。
     * @return
     */
    @Execute(validator = false ,removeActionForm = true)
    public String reset() {
    	LOGGER.debug("start-----.");
    	if(!loginChecker.isLogin(userDto)){
    		return "/login?redirect=true";
    	}
    	LOGGER.debug("end-----.");
    	return "/tbShinseiSearch?redirect=true";
    }

    /**
     * 申請条件画面へリダイレクトします。
     * @return
     */
    @Execute(validator = false)
    public String redirectSaerch(){
    	return "/tbShinseiSearch/index?redirect=true";
    }


}