package jp.savethememory.manage.service;

import java.util.List;
import java.util.ResourceBundle;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.seasar.framework.beans.util.BeanMap;
import org.seasar.framework.beans.util.Beans;

import jp.savethememory.manage.dto.UserDto;
import jp.savethememory.manage.entity.TbShinsei;
import jp.savethememory.manage.form.TbShinseiSearchForm;
import jp.savethememory.manage.util.CommonDefine;

/**
 * 申請テーブルに対する操作を行うクラス
 *
 */
public class TbShinseiService extends AbstractService<TbShinsei> {

	public UserDto userDto;

	protected Log LOGGER = LogFactory.getLog(TbShinseiService.class);
	private ResourceBundle PROPERTIES = ResourceBundle.getBundle(CommonDefine.PROPERTIES_STMM_FILE_NAME);

    public TbShinsei findById(Long shinseiNo) {
    	LOGGER.debug("start-----.");
    	TbShinsei shinsei = select().id(shinseiNo).getSingleResult();
    	LOGGER.debug("end-----.");
    	return shinsei;
    }


	/**
	 * 検索条件に 該当する申請情報を返します。
	 * tbShinseiSearchFormのデータから、BeanMapを使って検索します。
	 * @param tbShinseiSearchForm 検索条件
	 * @return List<TbShinsei>　該当する申請情報を返します。
	 */
	public List<TbShinsei> getTbShinseiItems(TbShinseiSearchForm tbShinseiSearchForm) {
		LOGGER.debug("start-----.");

		BeanMap map = getSearchMap(tbShinseiSearchForm);

		int max = Integer.parseInt(PROPERTIES.getString(CommonDefine.PROPERTIES_STMM_MAX_NUMBER_OF_SHINSEI_IN_A_PAGE));
		List<TbShinsei> results = jdbcManager
			.from(TbShinsei.class)
			.where(map)
			.orderBy("shinseiNo desc")
			.limit(max)
			.offset((tbShinseiSearchForm.page - 1) * max)
			.getResultList();

		LOGGER.debug("end-----.");
		return results;
	}

	/**
	 * 検索条件に 該当する申請の件数を返します。
	 */
	public long getTbShinseiItemsCount(TbShinseiSearchForm tbShinseiSearchForm) {
		LOGGER.debug("start-----.");

		BeanMap map = getSearchMap(tbShinseiSearchForm);

		long results = 0;
		results = jdbcManager
			.from(TbShinsei.class)
			.where(map)
			.getCount();

		LOGGER.debug("end-----.");
		return results;
	}

	/**
	 * 検索条件を設定したMAPを返します。
	 */
	private BeanMap getSearchMap(TbShinseiSearchForm tbShinseiSearchForm){
		BeanMap map = null;

tbShinseiSearchForm.search_shichousonCode_EQ = userDto.shichousonCode;

		if(tbShinseiSearchForm.shinseiNoRadio.equals("1")){
			if(("").equals(tbShinseiSearchForm.search_shinseiNo_EQ)){
				tbShinseiSearchForm.search_shinseiNo_EQ = null;
			}
			tbShinseiSearchForm.search_shinseiNo_LE = null;
			tbShinseiSearchForm.search_shinseiNo_GE = null;
		}else if(tbShinseiSearchForm.shinseiNoRadio.equals("2")){
			tbShinseiSearchForm.search_shinseiNo_EQ = null;
			if(("").equals(tbShinseiSearchForm.search_shinseiNo_LE)){
				tbShinseiSearchForm.search_shinseiNo_LE = null;
			}
			if(("").equals(tbShinseiSearchForm.search_shinseiNo_GE)){
				tbShinseiSearchForm.search_shinseiNo_GE = null;
			}
		}

		//%自分で追加しなきゃだめ？フレームワークのバグ？
		if(("").equals(tbShinseiSearchForm.shinseiNameSei)){
			tbShinseiSearchForm.search_shinseiNameSei_LIKE = null;
		}else{
			tbShinseiSearchForm.search_shinseiNameSei_LIKE =
				"%"+ tbShinseiSearchForm.shinseiNameSei + "%";
		}

		if(("").equals(tbShinseiSearchForm.shinseiNameMei)){
			tbShinseiSearchForm.search_shinseiNameMei_LIKE = null;
		}else{
			tbShinseiSearchForm.search_shinseiNameMei_LIKE =
				"%"+ tbShinseiSearchForm.shinseiNameMei + "%";
		}

		if(("").equals(tbShinseiSearchForm.shinseiFuriganaSei)){
			tbShinseiSearchForm.search_shinseiFuriganaSei_LIKE = null;
		}else{
			tbShinseiSearchForm.search_shinseiFuriganaSei_LIKE
			= "%"+ tbShinseiSearchForm.shinseiFuriganaSei + "%";
		}


		if(("").equals(tbShinseiSearchForm.shinseiFuriganaMei)){
			tbShinseiSearchForm.search_shinseiFuriganaMei_LIKE = null;
		}else{
			tbShinseiSearchForm.search_shinseiFuriganaMei_LIKE
			= "%"+ tbShinseiSearchForm.shinseiFuriganaMei + "%";
		}

		/** 電話番号 */
		if(("").equals(tbShinseiSearchForm.search_telNo1_EQ)){
			tbShinseiSearchForm.search_telNo1_EQ = null;
		}
		if(("").equals(tbShinseiSearchForm.search_telNo2_EQ)){
			tbShinseiSearchForm.search_telNo2_EQ = null;
		}
		if(("").equals(tbShinseiSearchForm.search_telNo3_EQ)){
			tbShinseiSearchForm.search_telNo3_EQ = null;
		}

//		if(("").equals(tbShinseiSearchForm.search_birthday_EQ)){
//			tbShinseiSearchForm.search_birthday_EQ = null;
//		}
//
//		if(("").equals(tbShinseiSearchForm.search_sex_EQ)){
//			tbShinseiSearchForm.search_sex_EQ = null;
//		}
		/**
		 * 住民票住所
		 */
		if(("").equals(tbShinseiSearchForm.search_juuminhyouJushoZipCode_EQ)){
		tbShinseiSearchForm.search_juuminhyouJushoZipCode_EQ = null;
		}

//		if(("").equals(tbShinseiSearchForm.search_juuminhyouJushoPrefecture_EQ)){
//			tbShinseiSearchForm.search_juuminhyouJushoPrefecture_EQ = null;
//		}
//		if(("").equals(tbShinseiSearchForm.juuminhyouJushoCity)){
//			tbShinseiSearchForm.search_juuminhyouJushoCity_LIKE = null;
//		}else{
//			tbShinseiSearchForm.search_juuminhyouJushoCity_LIKE
//			= "%"+ tbShinseiSearchForm.juuminhyouJushoCity + "%";
//		}
//
//
//		if(("").equals(tbShinseiSearchForm.juuminhyouJushoBantiGou)){
//			tbShinseiSearchForm.search_juuminhyouJushoBantiGou_LIKE = null;
//		}else{
//			tbShinseiSearchForm.search_juuminhyouJushoBantiGou_LIKE
//			= "%"+ tbShinseiSearchForm.juuminhyouJushoBantiGou + "%";
//		}
//
//		if(("").equals(tbShinseiSearchForm.juuminhyouJushoTatemono)){
//			tbShinseiSearchForm.search_juuminhyouJushoTatemono_LIKE = null;
//		}else{
//			tbShinseiSearchForm.search_juuminhyouJushoTatemono_LIKE
//			= "%"+ tbShinseiSearchForm.juuminhyouJushoTatemono + "%";
//		}


		/**
		 * 現住所
		 */
		if(("").equals(tbShinseiSearchForm.search_genJushoJushoZipCode_EQ)){
			tbShinseiSearchForm.search_genJushoJushoZipCode_EQ = null;
		}

//		if(("").equals(tbShinseiSearchForm.search_genJushoJushoPrefecture_EQ)){
//			tbShinseiSearchForm.search_genJushoJushoPrefecture_EQ = null;
//		}
//		if(("").equals(tbShinseiSearchForm.genJushoJushoCity)){
//			tbShinseiSearchForm.search_genJushoJushoCity_LIKE = null;
//		}else{
//			tbShinseiSearchForm.search_genJushoJushoCity_LIKE
//			= "%"+ tbShinseiSearchForm.genJushoJushoCity + "%";
//		}
//
//		s
//		if(("").equals(tbShinseiSearchForm.genJushoJushoBantiGou)){
//			tbShinseiSearchForm.search_genJushoJushoBantiGou_LIKE = null;
//		}else{
//			tbShinseiSearchForm.search_genJushoJushoBantiGou_LIKE
//			= "%"+ tbShinseiSearchForm.genJushoJushoBantiGou + "%";
//		}
//
//		if(("").equals(tbShinseiSearchForm.genJushoJushoTatemono)){
//			tbShinseiSearchForm.search_genJushoJushoTatemono_LIKE = null;
//		}else{
//			tbShinseiSearchForm.search_genJushoJushoTatemono_LIKE
//			= "%"+ tbShinseiSearchForm.genJushoJushoTatemono + "%";
//		}
		/** その他 */
		if(("").equals(tbShinseiSearchForm.mailAddress)){
			tbShinseiSearchForm.search_mailAddress_LIKE = null;
		}else{
			tbShinseiSearchForm.search_mailAddress_LIKE = "%"+ tbShinseiSearchForm.mailAddress + "%";

		}

//		if(("").equals(tbShinseiSearchForm.shinseishaComment)){
//			tbShinseiSearchForm.search_shinseishaComment_LIKE = null;
//		}else{
//			tbShinseiSearchForm.search_shinseishaComment_LIKE
//			= "%"+ tbShinseiSearchForm.shinseishaComment + "%";
//		}

		if(("").equals(tbShinseiSearchForm.search_shinseiStatus_EQ)){
			tbShinseiSearchForm.search_shinseiStatus_EQ = null;
		}
		if(("").equals(tbShinseiSearchForm.search_digitalStatus_EQ)){
			tbShinseiSearchForm.search_digitalStatus_EQ = null;
		}

//		if(("").equals(tbShinseiSearchForm.search_hikiwatashiSyudan_EQ)){
//			tbShinseiSearchForm.search_hikiwatashiSyudan_EQ = null;
//		}
//
//		if(("").equals(tbShinseiSearchForm.search_staffComment_LIKE)){
//			tbShinseiSearchForm.search_staffComment_LIKE = null;
//		}
//
		//prefixにsearch_が付いているものだけを検索対象とします。
		map = Beans.createAndCopy(BeanMap.class, tbShinseiSearchForm).prefix("search_").execute();

		return map;
	}

	/**
	 * 申請データがないか探す。条件はQuanpIDの取得に成功したもののみ。
	 * @param mailAddress
	 * @return
	 */
	public TbShinsei  findByMailAddress(String mailAddress) {
    	LOGGER.debug("start-----.");

    	TbShinsei shinsei = null;
    	List<TbShinsei>  shinseiList
    		= jdbcManager.from(TbShinsei.class)
    		.where("mailAddress = ? and quanpIdStatus = ?",
    				mailAddress,
    				Integer.valueOf(CommonDefine.QUANP_ID_STATUS_SUCCESS))
    		.orderBy("shinseiNo")
    		.getResultList();

    	if(shinseiList != null && shinseiList.size() != 0){
    		shinsei = shinseiList.get(0);
    	}

    	LOGGER.debug("end-----.");
    	return shinsei;
	}


	/**
	 *  レコードのロックは可能だが、nowait部分は有効に動かず、Exceptionが発生せずに、待ってしまう。
	 * @param shinseiNo
	 * @return
	 */
	public TbShinsei findByIdLock(Long shinseiNo) {
	   	LOGGER.debug("start-----.");
    	TbShinsei shinsei = jdbcManager.selectBySql(TbShinsei.class,
    			"select * from tb_shinsei where shinsei_no = ? for update nowait", shinseiNo)
    	.getSingleResult();
    	LOGGER.debug("end-----.");
    	return shinsei;
	}


	/**
	 * 指定された申請NOのdigitalStatusを指定された状態に更新する。
	 *
	 * @param shinseiNo
	 * @param DigitalStatus
	 */
	public void updateDigitalStatus(String shinseiNo,
			int DigitalStatus) {
		LOGGER.debug("start-----.");
		String sql = "update tb_shinsei set digital_status = ? where shinsei_no = ?";
		int count = jdbcManager
				.updateBySql(sql, Integer.class, Long.class)
				.params(Integer.valueOf(DigitalStatus),
						Long.valueOf(shinseiNo)).execute();
		if (count == 0) {
			LOGGER.error("error"+" shinseiNo:"
					+ shinseiNo);
		}
		LOGGER.debug("end-----.");
	}




}