package jp.savethememory.manage.service;

import java.util.List;

import jp.savethememory.manage.entity.TbSyashin;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.seasar.extension.jdbc.SqlBatchUpdate;

/**
 * 写真テーブルに対する操作を行うクラス
 *
 *
 */
public class TbSyashinService extends AbstractService<TbSyashin> {

	protected Log LOGGER = LogFactory.getLog(TbSyashinService.class);
	private static final String COMMENT_PREFIX = " [STMP";
	private static final String COMMENT_SUFFIX = "] ";

	public TbSyashin findById(Long contentsId) {
		return select().id(contentsId).getSingleResult();
	}

	/**
	 * 申請Noで写真テーブルからレコードを探します。
	 *
	 * @param shinseiNo
	 * @return
	 */
	public List<TbSyashin> findByShinseiNo(String shinseiNo) {
		LOGGER.debug("start-----.");
		List<TbSyashin> results = null;
		if (shinseiNo == null || ("").equals(shinseiNo)) {
			return results;
		}

		results = jdbcManager.from(TbSyashin.class)
				.where("shinseiNo = ?", Long.valueOf(shinseiNo))
				.orderBy("fileName asc").getResultList();
		LOGGER.debug("end-----.");
		return results;
	}

	/**
	 * 申請Noとファイル名で写真テーブルからレコードを探します。
	 *
	 * @param shinseiNo
	 * @return
	 */
	public List<TbSyashin> findByShinseiNoAndFileName(String shinseiNo,
			String fileName) {
		LOGGER.debug("start-----.");
		List<TbSyashin> results = null;
		if (shinseiNo == null || ("").equals(shinseiNo)) {
			return results;
		}

		results = jdbcManager
				.from(TbSyashin.class)
				.where("shinseiNo = ? and fileName like ?",
						Long.valueOf(shinseiNo), '%' + fileName + '%')
				.orderBy("fileName asc").getResultList();
		LOGGER.debug("end-----.");
		return results;
	}

	/**
	 * コンテンツIDで写真テーブルからレコードを探します。
	 *
	 * @param shinseiNo
	 * @return
	 */
	public TbSyashin findByContentId(String contentId) {
		LOGGER.debug("start-----.");
		TbSyashin results = null;
		if (contentId == null || ("").equals(contentId)) {
			return results;
		}

		results = jdbcManager.from(TbSyashin.class).id(contentId)
				.getSingleResult();
		LOGGER.debug("end-----.");
		return results;
	}

	/**
	 * 指定された申請Noの中から、 原本引渡済フラグがtrueに なっているものを選択します。
	 *
	 * @param shinseiNo
	 *            申請No
	 * @return
	 */
	public List<TbSyashin> findReturnFlgisTrue(String shinseiNo) {
		LOGGER.debug("start-----.");
		List<TbSyashin> results = null;
		if (shinseiNo == null || ("").equals(shinseiNo)) {
			return results;
		}

		results = jdbcManager
				.from(TbSyashin.class)
				.where("shinseiNo = ? and returnFlg = ?",
						Long.valueOf(shinseiNo), true).orderBy("fileName")
				.getResultList();
		LOGGER.debug("end-----.");
		return results;
	}

	/**
	 * 指定された申請Noの写真の返却フラグを返却済みに更新する。
	 *
	 * @param shinseiNo
	 */
	public int updateRetuneFlgtoTrue(String shinseiNo) {
		LOGGER.debug("start-----.");
		int ret = 0;

		int[] count = jdbcManager
				.updateBatchBySql(
						"update tb_syashin set return_flg = ? where shinsei_no = ?",
						Boolean.class, Long.class)
				.params(Boolean.valueOf(true), Long.valueOf(shinseiNo))
				.execute();
		if (count != null && count.length != 0) {
			ret = count[0];
			LOGGER.debug("update count:" + count[0]);
		}
		LOGGER.debug("end-----.");
		return ret;
	}

	/**
	 * 指定された申請Noの写真のサマリーに申請NOを登録する
	 */
	public void setSummary(String shinseiNo) {

		String shinseiNoSummary = COMMENT_PREFIX + shinseiNo + COMMENT_SUFFIX;

		// 更新前のコメントを取得するためにDBからTbSyashinを取得
		List<TbSyashin> targetTbSyashinList = findByShinseiNo(shinseiNo);

		for (TbSyashin targetTbSyashin : targetTbSyashinList) {
			String inputStr = null;
			if (targetTbSyashin.getSummary().isEmpty()) {
				// 更新前のコメントが入力されていない場合
				inputStr = shinseiNoSummary;
			} else {
				// 更新前からコメントが入力されていた場合
				inputStr = targetTbSyashin.getSummary() + "\n"
						+ shinseiNoSummary;
			}

			// コメントを更新

			int count = jdbcManager
					.updateBySql(
							"update tb_syashin set summary = ? where contents_id = ?",
							String.class, Long.class)
					.params(String.valueOf(inputStr),
							Long.valueOf(targetTbSyashin.getContentsId()))
					.execute();

			if (count == 0) {
				LOGGER.debug(targetTbSyashin.getContentsId() + "サマリー更新失敗");
			}
		}
	}

	/**
	 * 指定された申請Noの写真のquanpUploadStatusを指定された状態に更新する。
	 *
	 * @param shinseiNo
	 * @param quanpUploadStatusNeed
	 */
	public void updateQuanpUploadStatus(String shinseiNo,
			String quanpUploadStatus) {
		LOGGER.debug("start-----.");
		String sql = "update tb_syashin set quanp_upload_status = ? where shinsei_no = ?";
		int[] count = jdbcManager
				.updateBatchBySql(sql, Integer.class, Long.class)
				.params(Integer.valueOf(quanpUploadStatus),
						Long.valueOf(shinseiNo)).execute();
		if (count != null && count.length != 0) {
			LOGGER.debug("update count:" + count[0] + " quanpUploadStatus:"
					+ quanpUploadStatus);
		}
		LOGGER.debug("end-----.");
	}

	/**
	 * アルバムの枚数を取得する
	 */
	public int getSyashinCountInAlbum(String titlePrefix) {
		LOGGER.debug("start-----.");
		int count = 0;
		count = (int) jdbcManager.from(TbSyashin.class)
				.where("fileName like ? ", '%' + titlePrefix + '%').getCount();
		LOGGER.debug("end-----.");
		return count;
	}

	/**
	 * バッチ更新を実行する
	 *
	 */
	public void updateExcludeFlg(List<TbSyashin> syashinList, String[] trueIds) {
		LOGGER.debug("start-----.");

		SqlBatchUpdate batchUpdate = jdbcManager.updateBatchBySql(
				"update tb_syashin set exclude_flg = ? where contents_id = ?",
				Boolean.class, Long.class);

		for (TbSyashin syashin : syashinList) {
			boolean exclude = false;
			for (String id : trueIds) {
				if (syashin.getContentsId().toString().equals(id)) {
					exclude = true;
					break;
				}
			}
			if (exclude) {
				batchUpdate.params(true, syashin.getContentsId());
			} else {
				batchUpdate.params(false, syashin.getContentsId());
			}
		}

		int[] count = batchUpdate.execute();

		if (count != null && count.length != 0) {
			LOGGER.debug("update count:" + count.length);
		}
		LOGGER.debug("end-----.");
	}

	/**
	 * 除外対象となっている写真を申請対象から外す
	 */
	public void updateToExcludeFlgIsTrue(String shinseiNo) {
		LOGGER.debug("start-----.");
		String sql = "update tb_syashin set shinsei_no = null, return_flg = null"
				+ ", quanp_upload_status = null , public_flg = null , exclude_flg = null "
				+ "where shinsei_no = ? and exclude_flg is true";
		int[] count = jdbcManager.updateBatchBySql(sql, Long.class)
				.params(Long.valueOf(shinseiNo)).execute();
		if (count != null && count.length != 0) {
			LOGGER.debug("update count:" + count[0]);
		}
		LOGGER.debug("end-----.");
	}

}