package jp.savethememory.action;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.util.ResourceBundle;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import jp.savethememory.dto.UserDto;
import jp.savethememory.service.StorageS3Service;
import jp.savethememory.util.CommonDefine;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.lf5.util.StreamUtils;
import org.seasar.framework.aop.annotation.Trace;
import org.seasar.struts.annotation.Execute;

@Trace
public class ImageGetAction {
	public String contentId;
	@Resource
	protected UserDto userDto;
	@Resource
	protected HttpServletResponse response;

	protected Log LOGGER = LogFactory.getLog(ImageGetAction.class);

	/**
	 * サムネイル画像を取得します。ディレクトリに存在しない場合はquanpから取得します。
	 * 
	 * @return
	 * @throws Exception
	 */
	@Execute(validator = false, urlPattern = "getThumbnail/{contentId}")
	public String getThumbnail() throws Exception {
		if (userDto.account == null || userDto.account == "") {
			LOGGER.error("ログインしていない");
			throw new Exception();
		}

		ResourceBundle bundle = null;
		bundle = ResourceBundle
				.getBundle(CommonDefine.PROPERTIES_STMP_FILE_NAME);

		try {

			StorageS3Service s3 = new StorageS3Service();

			// ファイルキーの作成
			String key = bundle
					.getString(CommonDefine.PROPERTIES_STMP_KEY_IMG_THUMBNAIL_PREFIX)
					+ contentId
					+ bundle.getString(CommonDefine.PROPERTIES_STMP_KEY_IMG_SUFFIX);

			InputStream in = s3.downloadFile(userDto.account, key);

			responseImg(in);

		} catch (IOException e) {
			LOGGER.error(" , USER : " + userDto.account, e);
			LOGGER.error("サムネイル取得エラー：コンテントID" + contentId
					+ "のサムネイルを取得できませんでした。");

		}
		return null;
	}

	/**
	 * ビュー画像を取得します。ディレクトリに存在しない場合はquanpから取得します。
	 * 
	 * @return
	 * @throws Exception
	 */
	@Execute(validator = false, urlPattern = "getView/{contentId}")
	public String getView() throws Exception {
		if (userDto.account == null || userDto.account == "") {
			LOGGER.error("ログインしていない");
			throw new Exception();
		}
		ResourceBundle bundle = null;
		bundle = ResourceBundle
				.getBundle(CommonDefine.PROPERTIES_STMP_FILE_NAME);

		try {

			StorageS3Service s3 = new StorageS3Service();

			// ファイルキーを作成
			String key = bundle
					.getString(CommonDefine.PROPERTIES_STMP_KEY_IMG_VIEW_PREFIX)
					+ contentId
					+ bundle.getString(CommonDefine.PROPERTIES_STMP_KEY_IMG_SUFFIX);

			InputStream in = s3.downloadFile(userDto.account, key);

			responseImg(in);

		} catch (IOException e) {
			LOGGER.error(" , USER : " + userDto.account, e);
			LOGGER.error("ビュー取得エラー：コンテントID" + contentId + "のビューを取得できませんでした。");

		}
		return null;
	}

	/**
	 * レスポンスにイメージファイルを載せて返す
	 */
	private void responseImg(InputStream in) {
		ServletOutputStream sos = null;
		try {
			response.setContentType("image/jpeg");
			sos = response.getOutputStream();
			StreamUtils.copy(in, sos);

		} catch (IOException e) {
			LOGGER.debug(" , USER : " + userDto.account, e);
		} finally {
			IOUtils.closeQuietly(in);
			IOUtils.closeQuietly(sos);
		}

	}

}
