package jp.savethememory.action;

import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.UUID;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.savethememory.dto.DispContentDto;
import jp.savethememory.dto.UserDto;
import jp.savethememory.entity.SearchContentCriteria;
import jp.savethememory.entity.TbKen;
import jp.savethememory.entity.TbShichouson;
import jp.savethememory.entity.TbShiori;
import jp.savethememory.entity.TbSyashin;
import jp.savethememory.entity.TbSyashinList;
import jp.savethememory.entity.TbTag;
import jp.savethememory.facerecognize.VerifyException;
import jp.savethememory.form.PictureListForm;
import jp.savethememory.form.ShippingAddressForm;
import jp.savethememory.service.FacesearchService;
import jp.savethememory.service.NormalSearchService;
import jp.savethememory.service.StorageS3Service;
import jp.savethememory.service.StorageService;
import jp.savethememory.util.CommonDefine;
import net.arnx.jsonic.JSON;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.upload.MultipartRequestHandler;
import org.seasar.extension.jdbc.JdbcManager;
import org.seasar.extension.jdbc.where.SimpleWhere;
import org.seasar.framework.aop.annotation.Trace;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.ResponseUtil;

@Trace
public class PictureListAction {

	public JdbcManager jdbcManager;
	// 自治体表示用
	public String municipality;
	// 表示するコンテンツのリスト
	public List<DispContentDto> contentList;
	public int total = 0;
	public int totalPage = 0;
	public int displayStartNumber = 0;
	public String searchMessage = null;
	public String bookmarkMessage = null;
	public String[] dateErrMessage = null;
	public List<DispContentDto> albumContentList;
	public String verifyErrorMessage = null;
	public String searchConditionErrorMessage = null;

	@Resource
	@ActionForm
	protected PictureListForm pictureListForm;

	@Resource
	protected ShippingAddressForm shippingAddressForm;

	@Resource
	protected HttpServletResponse response;

	@Resource
	protected HttpServletRequest request;

	@Resource
	protected UserDto userDto;

	@Resource
	protected FacesearchService facesearchService;

	@Resource
	protected NormalSearchService normalSearchService;

	@Resource
	protected StorageService storageService;

	protected Log LOGGER = LogFactory.getLog(PictureListAction.class);

	private HashMap<String, Integer> albumContentsMap;
	// しおりを使った検索の場合における検索対象のファイル名
	private String bookmarkTargetFileName = null;

	private int NUMBER_OF_SMALL_PICTURE_IN_A_BLOCK = 4;
	private int NUMBER_OF_BIG_PICTURE_IN_A_BLOCK = 1;
	private ResourceBundle bundle = ResourceBundle
			.getBundle(CommonDefine.PROPERTIES_STMP_FILE_NAME);
	private final String INFO_NOT_SEARCHED_PICTURE = "下記条件に該当する写真はありませんでした。";
	private final String INFO_NOT_BOOKMARK = "指定された電話番号は登録されていません。";
	private final String ERROR_DATE_CONVERSE = "開始年月日と終了年月日の指定が逆です。";
	private final String ERROR_DATE_TO_NOT_ALL = "範囲指定終了年月日を全て指定してください。";
	private final String ERROR_DATE_FROM_NOT_ALL = "範囲指定開始年月日を全て指定してください。";

	private final int IMAGE_WIDTH_MAXSIZE = 940;

	/**
	 * 検索画面を表示します
	 * 
	 **/
	@Execute(validator = false)
	public String index() {
		this.pictureListForm.from = 0;
		this.pictureListForm.bookmarkKey = "";
		this.pictureListForm.tagList = new ArrayList<TbTag>();

		// quanpからではなくDBからタグを取得するように変更。
		// this.pictureListForm.tagListの型をList<Tag>からList<TbTag>に変更。
		this.pictureListForm.tagList = storageService
				.loadTagList(userDto.account);

		return "index.jsp";
	}

	/**
	 * 検索後、写真一覧を表示します。
	 */
	@Execute(validator = false)
	public String searchPictureList() {
		Boolean exceeded = (Boolean) request
				.getAttribute(MultipartRequestHandler.ATTRIBUTE_MAX_LENGTH_EXCEEDED);
		if (exceeded != null && exceeded.booleanValue()) {
			LOGGER.error("ファイルサイズが上限値5MBに越えました。");
			searchConditionErrorMessage = "サイズが5MB以内の写真をご利用ください。";
			return "faceSearch.jsp";
		}
		String retStr = "displayPictureList";

		verifyErrorMessage = null;

		// 検索画面から遷移の場合は、ページ番号を初期化する
		this.pictureListForm.page = 1;

		if (pictureListForm.searchType == 0) { // 普通検索
			// 日付の妥当性チェック
			if (this.pictureListForm.searchCondition == 0 && !isDateValid()) {
				retStr = "index.jsp";
			}
		} else {
			try {
				long starttime = Calendar.getInstance().getTimeInMillis();
				LOGGER.info("[SearchTime] Face Search Start: " + starttime);
				if (pictureListForm.pictureFrom == 0) { // アップロードした写真を使う
					if (pictureListForm.facePicture == null
							|| pictureListForm.facePicture.getFileName() == null
							|| pictureListForm.facePicture.getFileName()
									.equals("")) {
						searchConditionErrorMessage = "ファイルを選択してください";
						return "faceSearch.jsp";
					}
					pictureListForm.verifyResults = facesearchService
							.findPictureListByLocalFace(
									pictureListForm.facePicture.getFileData(),
									userDto.account);
				} else { // サーバー上の写真を使う
					if (pictureListForm.featureFilePath.equals("")) {
						searchConditionErrorMessage = "顔を選択してください。";
						return "faceSearch.jsp";
					}
					pictureListForm.verifyResults = facesearchService
							.findPictureListByServerFace(
									pictureListForm.featureFilePath,
									userDto.account);
				}
				long stoptime = Calendar.getInstance().getTimeInMillis();
				LOGGER.info("[SearchTime] Face Search finish: " + stoptime);
				LOGGER.info("[SearchTime] Face Search cost time: "
						+ (stoptime - starttime));

				if (pictureListForm.verifyResults == null) {
					verifyErrorMessage = "顔が検出できませんでした。";
					return "faceSearch.jsp";
				} else if (pictureListForm.verifyResults.isEmpty()) {
					verifyErrorMessage = "該当する写真はありませんでした。";
					return "faceSearch.jsp";
				}
			} catch (FileNotFoundException e) {
				LOGGER.error("アップロードしたファイルが無効です。", e);
				verifyErrorMessage = "アップロードしたファイルが無効です,別の写真を使ってください。";
				retStr = "faceSearch.jsp";
			} catch (VerifyException e) {
				LOGGER.error("顔比較するときエラー発生した。", e);
				verifyErrorMessage = "顔比較が失敗しました,別の写真を使ってください。";
				retStr = "faceSearch.jsp";
			} catch (IOException e) {
				LOGGER.error("IOException", e);
				verifyErrorMessage = "顔比較が失敗しました,別の写真を使ってください。";
				retStr = "faceSearch.jsp";
			} catch (Exception e) {
				LOGGER.error("Exception", e);
				verifyErrorMessage = "顔比較が失敗しました,別の写真を使ってください。";
				retStr = "faceSearch.jsp";
			}

		}

		return retStr;
	}

	/**
	 * 写真一覧を表示します。 ソート順：ファイル名昇順
	 * 
	 * @return
	 */
	@Execute(validator = false)
	public String displayPictureList() {
		String returnJsp = "pictureList.jsp";

		// パンクズリスト用文言作成
		setBreadCrumbList();

		int numberOfContent = 0;

		// 画像の大きさ
		if (this.pictureListForm.pictureSize == 0) {
			numberOfContent = Integer
					.parseInt(bundle
							.getString(CommonDefine.PROPERTIES_STMP_KEY_NUMBER_OF_SMALL_CONTENT_AT_A_TIME));
		} else {
			numberOfContent = Integer
					.parseInt(bundle
							.getString(CommonDefine.PROPERTIES_STMP_KEY_NUMBER_OF_BIG_CONTENT_AT_A_TIME));
			returnJsp = "bigPictureList.jsp";
		}

		// しおり
		if (this.pictureListForm.selectedTitlePrefix.isEmpty()) {
			this.pictureListForm.selectedCount = 0;
		}

		TbSyashinList targetTbSyashinList = null;
		List<TbSyashin> targetContents = null;
		List<String> hasFaceContentIds = null;
		int totalResults = 0;
		if (pictureListForm.searchType == 0) { // 普通検索

			// DBから検索結果
			targetTbSyashinList = searchFromDb(numberOfContent);

			if (targetTbSyashinList != null
					&& targetTbSyashinList.getTotalResults() > 0) {
				totalResults = targetTbSyashinList.getTotalResults();
				targetContents = targetTbSyashinList.getTbSyashinList();
				hasFaceContentIds = facesearchService
						.receiveHasFaceContentIds(targetContents);
			}
		} else { // 顔検索
			targetContents = searchFromFace(numberOfContent);
			if (pictureListForm.verifyResults != null) {
				totalResults = pictureListForm.verifyResults.size();
			}
		}

		if (targetContents != null && targetContents.size() > 0) {
			// 表示するコンテンツの情報を設定
			setDisplayContentsInfo(targetContents, hasFaceContentIds);
			// ページング用情報をセット
			setPagingInfo(totalResults, numberOfContent,
					this.pictureListForm.page);
		} else {
			searchMessage = this.INFO_NOT_SEARCHED_PICTURE;
			returnJsp = "index.jsp";
		}

		// ブラウザキャッシュを無効にする処理(これが有効だとチェック状態、ハイライト状態などが記憶されてしまうことがある)
		HttpServletResponse response = ResponseUtil.getResponse();
		response.setDateHeader("Expires", -1);
		response.setHeader("Pragma", "no-cache");
		response.setHeader("Cache-Control", "no-cache");

		return returnJsp;
	}

	// searchFromQuanpからメソッド名を変更。
	private TbSyashinList searchFromDb(int numberOfContent) {
		// コンテンツの検索条件をセット
		SearchContentCriteria criteria = setCriteria();

		TbSyashinList tbSYashinList = normalSearchService.searchTbSyashinList(
				userDto.account, this.pictureListForm.searchCondition,
				criteria, 1, 1);

		if (tbSYashinList.getTotalResults() > 0) {
			// 一覧表示中にコンテンツが移動されページングの整合がとれなくなった場合の処理
			if (tbSYashinList.getTotalResults() <= (this.pictureListForm.page - 1)
					* numberOfContent) {
				if (tbSYashinList.getTotalResults() % numberOfContent == 0) {
					this.pictureListForm.page = tbSYashinList.getTotalResults()
							/ numberOfContent;
				} else {
					this.pictureListForm.page = tbSYashinList.getTotalResults()
							/ numberOfContent + 1;
				}
			}

			TbSyashinList targetTbSYashinList = getTargetContentList(criteria,
					numberOfContent);

			return targetTbSYashinList;

		} else {// 検索結果がない場合
			return null;
		}
	}

	private List<TbSyashin> searchFromFace(int numberOfContent) {
		// コンテンツの検索条件をセット

		if (pictureListForm.verifyResults != null
				&& pictureListForm.verifyResults.size() > 0) {
			int totalResults = pictureListForm.verifyResults.size();
			// 一覧表示中にコンテンツが移動されページングの整合がとれなくなった場合の処理
			if (totalResults <= (this.pictureListForm.page - 1)
					* numberOfContent) {
				if (totalResults % numberOfContent == 0) {
					this.pictureListForm.page = totalResults / numberOfContent;
				} else {
					this.pictureListForm.page = totalResults / numberOfContent
							+ 1;
				}
			}

			int fromIndex = (pictureListForm.page - 1) * numberOfContent;
			int toIndex = (pictureListForm.page) * numberOfContent;
			if (toIndex > totalResults) {
				toIndex = totalResults;
			}
			List<TbSyashin> contents = pictureListForm.verifyResults.subList(
					fromIndex, toIndex);
			return contents;
		} else {
			return null;
		}

	}

	/**
	 * 表示対象のコンテンツリストを取得する
	 */
	private TbSyashinList getTargetContentList(SearchContentCriteria criteria,
			int numberOfContent) {
		boolean isTargetPage = true;
		if (this.bookmarkTargetFileName != null) {
			// しおりから検索の場合、前回表示していたコンテンツが見つかるまで繰り返し検索を行う
			// コンテンツがリターンプレイスに移動した場合にはファイル名ソート順で１つ前のコンテンツが含まれるページを表示する
			isTargetPage = false;
		}

		TbSyashinList targetTbSYashinList = null;
		String prevStartFileName = null;
		String prevEndFileName = null;

		while (true) {
			// ソート順：ファイル名でコンテンツの検索を行う
			targetTbSYashinList = normalSearchService.searchTbSyashinList(
					userDto.account, this.pictureListForm.searchCondition,
					criteria, numberOfContent, this.pictureListForm.page);

			if (isTargetPage) {
				break;
			} else if (targetTbSYashinList != null) {
				// 先頭のコンテンツのファイル名
				String startFileName = targetTbSYashinList.getTbSyashinList()
						.get(0).getFileName();
				// 末尾のコンテンツのファイル名
				String endFileName = targetTbSYashinList.getTbSyashinList()
						.get(targetTbSYashinList.getTbSyashinList().size() - 1)
						.getFileName();

				if (startFileName.compareTo(this.bookmarkTargetFileName) <= 0
						&& endFileName.compareTo(this.bookmarkTargetFileName) >= 0) {
					// ファイル名のソート順で　先頭ファイル <= しおり情報ファイル <=
					// 末尾ファイル　の場合はこのページを表示対象ページとする
					break;
				} else if (prevStartFileName != null
						&& (endFileName.compareTo(this.bookmarkTargetFileName) < 0 && prevStartFileName
								.compareTo(this.bookmarkTargetFileName) > 0)) {
					// 末尾ファイル < しおり情報ファイル <
					// 前に処理をしたページの先頭ファイル　の場合はこのページを表示対象ページとする
					break;
				} else if (prevEndFileName != null
						&& (prevEndFileName
								.compareTo(this.bookmarkTargetFileName) < 0 && startFileName
								.compareTo(this.bookmarkTargetFileName) > 0)) {
					// 前に処理をしたページの末尾ファイル < しおり情報ファイル <
					// 先頭ファイル　の場合は１つ前のページを表示対象ページとする
					this.pictureListForm.page--;
					isTargetPage = true;
				} else if (startFileName.compareTo(this.bookmarkTargetFileName) > 0) {
					// １ページ目の場合はこのページを表示対象ページとする
					if (this.pictureListForm.page == 1) {
						break;
					}
					// 先頭ファイル > しおり情報ファイル　の場合は１つ前のページを検索する
					this.pictureListForm.page--;
					prevStartFileName = startFileName;
					prevEndFileName = null;
					// ただし、１つ前のページが１ページ目の場合は１ページ目を表示対象とする
					if (this.pictureListForm.page == 1) {
						isTargetPage = true;
					}
				} else if (endFileName.compareTo(this.bookmarkTargetFileName) < 0) {
					// 最終ページの場合はこのページを表示対象ページとする
					if (!targetTbSYashinList.hasNext()) {
						break;
					}
					// 末尾ファイル < しおり情報ファイル　の場合は１つ後のページを検索する
					this.pictureListForm.page++;
					prevEndFileName = endFileName;
					prevStartFileName = null;
				}

			} else {
				// コンテンツリストが無い場合はページを１減らして検索
				this.pictureListForm.page--;
				if (this.pictureListForm.page == 1) {
					isTargetPage = true;
				}
			}
		}

		return targetTbSYashinList;
	}

	/**
	 * 表示するコンテンツ情報を設定する
	 */
	private void setDisplayContentsInfo(List<TbSyashin> targetContents,
			List<String> hasFaceContentIds) {
		contentList = new ArrayList<DispContentDto>();
		this.albumContentsMap = new HashMap<String, Integer>();

		int count = 0;
		int anchorCount = 0;
		boolean isPrevAlbum = false;
		String prevPrefix = "";
		String tmpAlbumPrefix = null;

		int numberOfPiture = this.NUMBER_OF_SMALL_PICTURE_IN_A_BLOCK;
		if (this.pictureListForm.pictureSize == 1) {
			numberOfPiture = this.NUMBER_OF_BIG_PICTURE_IN_A_BLOCK;
		}

		if (targetContents != null && !targetContents.isEmpty()) {
			this.pictureListForm.displayStartFileName = targetContents.get(0)
					.getFileName();

			// TODO:
			StringBuilder selectSb = new StringBuilder();
			for (int i = 0, n = targetContents.size(); i < n; i++) {
				TbSyashin content = targetContents.get(i);
				DispContentDto dcont = new DispContentDto();
				dcont.id = String.valueOf(content.getContentsId());
				dcont.published = content.getPublishedDay();
				dcont.summary = content.getSummary();
				if (hasFaceContentIds == null) {
					dcont.isHasFace = true;
				} else if (hasFaceContentIds.contains(dcont.id)) {
					dcont.isHasFace = true;
				} else {
					dcont.isHasFace = false;
				}

				dcont.canSelect = true;
				dcont.albumContentsCount = 0;
				dcont.isPublicPhoto = false;

				String[] titleInfo = getTitleInfo(content.getFileName());

				if (titleInfo != null) {
					dcont.title = titleInfo[0];
					dcont.titlePrefix = titleInfo[1];
				} else {
					// ファイル名が正しくない場合
					dcont.title = content.getFileName();
					dcont.titlePrefix = dcont.title;
					dcont.canSelect = false;
					LOGGER.warn("FILE NAME IS NOT CORRECT. CONTENT ID:"
							+ content.getContentsId() + " FILE NAME:"
							+ dcont.title);
				}

				/**
				 * 表示時にバラの場合は４枚(表示サイズ小の場合。大の場合は１枚ごと)ごと、アルバムの場合はアルバムごとで段落とするための処理
				 **/
				boolean isAlbum = false;
				if (dcont.titlePrefix.equals(tmpAlbumPrefix)) {
					// 1つ前のコンテンツとプレフィックスが等しい場合、アルバム
					isAlbum = true;
				} else if (i + 1 < n) {
					// 1つ前のコンテンツとプレフィックスが異なる場合
					TbSyashin nextContent = null;
					nextContent = targetContents.get(i + 1);
					String nextPrefix = "";
					if (nextContent != null) {
						String[] tInfo = getTitleInfo(nextContent.getFileName());

						if (tInfo != null) {
							nextPrefix = tInfo[1];
						}
					}

					if (dcont.titlePrefix.equals(nextPrefix)) {
						// 1つ後ろのコンテンツとプレフィックスが等しい場合、アルバム
						tmpAlbumPrefix = dcont.titlePrefix;
						isAlbum = true;
					} else {
						tmpAlbumPrefix = null;
					}
				}

				if (isAlbum != isPrevAlbum || count == 0) {
					// 前処理コンテンツの種類（バラ/アルバム）が異なる場合
					dcont.anchorCount = anchorCount++;
					count = 1;
				} else if (!isAlbum && count == numberOfPiture) {
					// バラの場合かつバラが４枚続いた場合
					dcont.anchorCount = anchorCount++;
					count = 1;
				} else if (isAlbum && !prevPrefix.equals(dcont.titlePrefix)) {
					// 前処理もアルバムかつ異なるアルバムの場合
					dcont.anchorCount = anchorCount++;
					count = 1;
				} else {
					dcont.anchorCount = -1;
					count++;
				}

				prevPrefix = dcont.titlePrefix;
				isPrevAlbum = isAlbum;
				/***********************************************/

				if (dcont.canSelect) {
					// TODO:
					selectSb.append(" contents_id = ");
					selectSb.append(content.getContentsId());
					selectSb.append(" or");

					// TbSyashin syashin =
					// jdbcManager.from(TbSyashin.class).id(content.getId()).getSingleResult();
					//
					// if(syashin != null){
					// //公開写真（原本無し）の判断
					// if(syashin.getPublicFlg() != null &&
					// syashin.getPublicFlg()){
					// dcont.isPublicPhoto = syashin.getPublicFlg();
					// //公開写真の場合は常に選択可能かつ１枚ものとして扱う
					// dcont.canSelect = true;
					// dcont.titlePrefix = getTitlePrefix(dcont.title ,
					// CommonDefine.FILE_NAME_EXTENSION_LENGTH);
					// dcont.albumContentsCount = 1;
					// } else {
					// //申請番号ありの場合は申請中
					// if(syashin.getShinseiNo() != null){
					// dcont.canSelect = false;
					// }
					// dcont.albumContentsCount =
					// getAlbumContentsCount(dcont.titlePrefix ,
					// syashin.getShinseiNo());
					// }
					// } else {
					// //DBにデータ無しの場合は表示側では選択ボタンを非表示
					// dcont.canSelect = false;
					// }
				}

				contentList.add(dcont);
			}

			if (selectSb.length() > 0) {
				selectSb.delete(selectSb.length() - 3, selectSb.length());

				List<TbSyashin> syashinList = jdbcManager.from(TbSyashin.class)
						.where(selectSb.toString()).getResultList();

				TbSyashin syashin;
				for (DispContentDto dcont : contentList) {
					if (dcont.canSelect) {
						syashin = null;
						for (TbSyashin s : syashinList) {
							if (s.getContentsId().toString().equals(dcont.id)) {
								syashin = s;
								break;
							}
						}

						if (syashin != null) {
							// 公開写真（原本無し）の判断
							if (syashin.getPublicFlg() != null
									&& syashin.getPublicFlg()) {
								dcont.isPublicPhoto = syashin.getPublicFlg();
								// 公開写真の場合は常に選択可能かつ１枚ものとして扱う
								dcont.canSelect = true;
								dcont.titlePrefix = getTitlePrefix(dcont.title,
										CommonDefine.FILE_NAME_EXTENSION_LENGTH);
								dcont.albumContentsCount = 1;
							} else {
								// 申請番号ありの場合は申請中
								if (syashin.getShinseiNo() != null) {
									dcont.canSelect = false;
								}
								dcont.albumContentsCount = getAlbumContentsCount(
										dcont.titlePrefix,
										syashin.getShinseiNo());
							}
						} else {
							// DBにデータ無しの場合は表示側では選択ボタンを非表示
							dcont.canSelect = false;
						}
					}
				}
			}

		}
	}

	/**
	 * STMPシステムで利用するタイトル、タイトルプレフィックスを取得 ファイル名不正の場合はnullを返す
	 * **/
	private String[] getTitleInfo(String baseTitle) {
		String[] ret = new String[2];

		if (CommonDefine.FILE_NAME_PATTERN1.matcher(baseTitle).find()) {
			// ファイル名の先頭が日付の場合は削除して表示
			ret[0] = baseTitle
					.substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH);
			ret[1] = getTitlePrefix(ret[0],
					CommonDefine.FILE_NAME_SUFFIX_LENGTH);
		} else if (CommonDefine.FILE_NAME_PATTERN2.matcher(baseTitle).find()) {
			// ファイル名の先頭が日付かつ８枚ものの場合
			ret[0] = baseTitle
					.substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH);
			ret[1] = getTitlePrefix(ret[0],
					CommonDefine.FILE_NAME_EIGHT_SUFFIX_LENGTH);
		} else if (CommonDefine.FILE_NAME_PATTERN3.matcher(baseTitle).find()) {
			// ファイル名の先頭が日付ではない場合
			ret[0] = baseTitle;
			ret[1] = getTitlePrefix(ret[0],
					CommonDefine.FILE_NAME_SUFFIX_LENGTH);
		} else {
			// //ファイル名が正しくない場合
			ret = null;
		}

		return ret;
	}

	/**
	 * しおりを用いて一覧を表示する
	 */
	@Execute(validator = true, input = "index.jsp")
	public String searchPictureListWithBookmark() {
		String retStr = "displayPictureList";

		// DBからしおり情報を取得
		TbShiori shiori = jdbcManager.from(TbShiori.class)
				.id(this.pictureListForm.bookmarkKey).getSingleResult();
		if (shiori == null) {
			bookmarkMessage = this.INFO_NOT_BOOKMARK;
			retStr = "index.jsp";
		} else {
			this.pictureListForm.page = shiori.getPage();

			if (shiori.getFromDate() != null || shiori.getToDate() != null) {
				this.pictureListForm.searchCondition = 0;
				String[] fromDate = shiori.getFromDate().trim().split(";");
				String[] toDate = shiori.getToDate().trim().split(";");
				this.pictureListForm.fromYear = Integer.parseInt(fromDate[0]);
				this.pictureListForm.fromMonth = Integer.parseInt(fromDate[1]);
				this.pictureListForm.fromDay = Integer.parseInt(fromDate[2]);
				this.pictureListForm.toYear = Integer.parseInt(toDate[0]);
				this.pictureListForm.toMonth = Integer.parseInt(toDate[1]);
				this.pictureListForm.toDay = Integer.parseInt(toDate[2]);
				this.pictureListForm.freeText = "";
				this.pictureListForm.tag = "";
			} else if (shiori.getFreeText() != null) {
				this.pictureListForm.searchCondition = 1;
				this.pictureListForm.fromYear = -1;
				this.pictureListForm.fromMonth = -1;
				this.pictureListForm.fromDay = -1;
				this.pictureListForm.toYear = -1;
				this.pictureListForm.toMonth = -1;
				this.pictureListForm.toDay = -1;
				this.pictureListForm.freeText = shiori.getFreeText().trim();
				this.pictureListForm.tag = "";
			} else {
				this.pictureListForm.searchCondition = 2;
				this.pictureListForm.fromYear = -1;
				this.pictureListForm.fromMonth = -1;
				this.pictureListForm.fromDay = -1;
				this.pictureListForm.toYear = -1;
				this.pictureListForm.toMonth = -1;
				this.pictureListForm.toDay = -1;
				this.pictureListForm.freeText = "";
				this.pictureListForm.tag = shiori.getTag().trim();
			}

			this.pictureListForm.pictureSize = shiori.getPictureSize();
			this.bookmarkTargetFileName = shiori.getFileName();
		}

		return retStr;
	}

	/**
	 * 全選択解除．(枚数のクリア、除外写真のクリア)
	 */
	@Execute(validator = false)
	public String clearSelected() {
		this.pictureListForm.selectedTitlePrefix = "";
		this.pictureListForm.selectedCount = 0;
		this.pictureListForm.excludeContentIdList = "";

		// ブラウザキャッシュを無効にする処理
		HttpServletResponse response = ResponseUtil.getResponse();
		response.setDateHeader("Expires", -1);
		response.setHeader("Pragma", "no-cache");
		response.setHeader("Cache-Control", "no-cache");

		ResponseUtil.write("0");
		return null;
	}

	/**
	 * 入力されたコメントをDBに反映
	 */
	@Execute(validator = false)
	public String setSummary() {
		String inputStr = "";

		int contentsId = Integer
				.parseInt(this.pictureListForm.inputSummaryTargetContent);

		// 更新前のコメントを取得するためにDBからTbSyashinを取得
		TbSyashin targetTbSyashin = normalSearchService
				.searchTbSyashin(contentsId);

		if (targetTbSyashin.getSummary().isEmpty()) {
			// 更新前のコメントが入力されていない場合
			inputStr = this.pictureListForm.inputSummary;

		} else {
			// 更新前からコメントが入力されていた場合
			inputStr = targetTbSyashin.getSummary() + "\n"
					+ this.pictureListForm.inputSummary;
		}

		targetTbSyashin.setSummary(inputStr);

		// コメントを更新
		storageService.updateTbSyashin(targetTbSyashin);

		ResponseUtil.write(inputStr);
		return null;
		// return displayPictureList();
	}

	/**
	 * 入力されたしおり用文字列が既に登録されているか確認する　
	 */
	@Execute(validator = false)
	public String existDuplicateBookmark() {
		String ret = "";

		TbShiori shiori = jdbcManager.from(TbShiori.class)
				.id(this.pictureListForm.inputBookmarkKey).getSingleResult();
		if (shiori != null) {
			ret = shiori.getUpdateDay().toString();
		}

		// ブラウザキャッシュを無効にする処理
		HttpServletResponse response = ResponseUtil.getResponse();
		response.setDateHeader("Expires", -1);
		response.setHeader("Pragma", "no-cache");
		response.setHeader("Cache-Control", "no-cache");

		ResponseUtil.write(ret);
		return null;
	}

	/**
	 * しおりを登録する
	 * 
	 */
	@Execute(validator = false)
	public String insertBookmark() {
		TbShiori shiori = new TbShiori();
		shiori.setKeyString(this.pictureListForm.inputBookmarkKey);
		setShioriData(shiori);

		jdbcManager.insert(shiori).execute();

		return null;
	}

	/**
	 * しおりを更新する
	 * 
	 */
	@Execute(validator = false)
	public String updateBookmark() {
		TbShiori shiori = jdbcManager.from(TbShiori.class)
				.id(this.pictureListForm.inputBookmarkKey).getSingleResult();

		if (shiori != null) {
			setShioriData(shiori);
			jdbcManager.update(shiori).execute();
		}

		return null;
	}

	/**
	 * 写真選択確認画面から戻る
	 * 
	 * @return
	 */
	@Execute(validator = false)
	public String backFromConfirm() {
		return displayPictureList();
	}

	/**
	 * しおり登録・更新用データをセットする
	 */
	private void setShioriData(TbShiori shiori) {
		shiori.setFileName(this.pictureListForm.displayStartFileName);

		if (this.pictureListForm.searchCondition == 0) {
			// 日付検索
			shiori.setFromDate(String.valueOf(this.pictureListForm.fromYear)
					+ ";" + String.valueOf(this.pictureListForm.fromMonth)
					+ ";" + String.valueOf(this.pictureListForm.fromDay));
			shiori.setToDate(String.valueOf(this.pictureListForm.toYear) + ";"
					+ String.valueOf(this.pictureListForm.toMonth) + ";"
					+ String.valueOf(this.pictureListForm.toDay));
			shiori.setFreeText(null);
		} else if (this.pictureListForm.searchCondition == 1) {
			// フリー入力
			shiori.setFromDate(null);
			shiori.setToDate(null);
			shiori.setFreeText(this.pictureListForm.freeText);
			shiori.setTag(null);
		} else {
			// タグ検索
			shiori.setFromDate(null);
			shiori.setToDate(null);
			shiori.setFreeText(null);
			shiori.setTag(this.pictureListForm.tag);
		}

		shiori.setPictureSize(this.pictureListForm.pictureSize);
		shiori.setPage(this.pictureListForm.page);
		shiori.setUpdateDay(new Timestamp(System.currentTimeMillis()));

	}

	/**
	 * 写真選択確認画面へ遷移する
	 */
	@Execute(validator = false)
	public String toConfirmSelectedPicture() {
		// パンクズリスト用文言作成
		setBreadCrumbList();

		// 確認画面用の写真リスト情報を設定する
		setDisplayContentsInfoForConfirm();

		return "confirm.jsp";
	}

	/**
	 * 確認画面に表示するコンテンツ情報を設定する
	 */
	private void setDisplayContentsInfoForConfirm() {
		contentList = new ArrayList<DispContentDto>();
		this.albumContentsMap = new HashMap<String, Integer>();

		if (!this.pictureListForm.selectedTitlePrefix.isEmpty()) {
			String[] checkedTitlePrefixList = this.pictureListForm.selectedTitlePrefix
					.split(";");

			for (String titlePrefix : checkedTitlePrefixList) {
				List<TbSyashin> syashinList = jdbcManager
						.from(TbSyashin.class)
						.where("file_name like ? and shinsei_no is null and public_flg is not true",
								'%' + titlePrefix + '%')
						.orderBy("fileName asc").getResultList();

				if (syashinList.isEmpty()) {
					// 公開対応の写真検索
					syashinList = jdbcManager
							.from(TbSyashin.class)
							.where("file_name like ? and public_flg is true",
									'%' + titlePrefix + '%').getResultList();
				}

				if (syashinList != null && syashinList.size() > 0) {
					// アルバムの場合、先頭のみを表示する
					TbSyashin syashin = syashinList.get(0);
					DispContentDto dcont = new DispContentDto();
					dcont.id = syashin.getContentsId().toString();
					dcont.published = new Date(syashin.getPublishedDay()
							.getTime());
					dcont.originalTitle = syashin.getFileName();

					String[] titleInfo = getTitleInfo(syashin.getFileName()
							.trim());
					dcont.title = titleInfo[0];
					dcont.titlePrefix = titleInfo[1];

					// 公開写真（原本無し）の判断
					if (syashin.getPublicFlg() != null
							&& syashin.getPublicFlg()) {
						dcont.isPublicPhoto = syashin.getPublicFlg();
						// 公開写真の場合は１枚ものとして扱う
						dcont.titlePrefix = getTitlePrefix(dcont.title,
								CommonDefine.FILE_NAME_EXTENSION_LENGTH);
						dcont.albumContentsCount = 1;
					} else {
						dcont.isPublicPhoto = false;
						dcont.albumContentsCount = syashinList.size();
					}

					contentList.add(dcont);
				}
			}

			// 表示順（ファイル名昇順）を設定
			Collections.sort(contentList, new Comparator<DispContentDto>() {
				public int compare(DispContentDto d1, DispContentDto d2) {
					return d1.originalTitle.compareTo(d2.originalTitle);
				}
			});
		}
	}

	/**
	 * アルバムの写真を表示する
	 */
	@Execute(validator = false, urlPattern = "displayAlbumPictureList/{contentId}&{canExclude}")
	public String displayAlbumPictureList() {
		TbSyashin syashin = null;
		syashin = jdbcManager.from(TbSyashin.class)
				.id(this.pictureListForm.contentId).getSingleResult();
		this.albumContentList = new ArrayList<DispContentDto>();

		if (syashin != null) {
			String titlePrefix = "";
			String[] titleInfo = getTitleInfo(syashin.getFileName().trim());
			if (titleInfo != null) {
				titlePrefix = titleInfo[1];
			}

			List<TbSyashin> syashinList = null;
			if (syashin.getShinseiNo() != null) {
				syashinList = jdbcManager
						.from(TbSyashin.class)
						.where("file_name like ? and shinsei_no = ?",
								'%' + titlePrefix + '%', syashin.getShinseiNo())
						.orderBy("fileName asc")
						// .limit(Integer.parseInt(bundle.getString(CommonDefine.PROPERTIES_STMP_KEY_NUMBER_OF_DISPLAY_ALBUM_CONTENT)))
						.getResultList();
			} else {
				syashinList = jdbcManager
						.from(TbSyashin.class)
						.where("file_name like ? and shinsei_no is null",
								'%' + titlePrefix + '%')
						.orderBy("fileName asc")
						// .limit(Integer.parseInt(bundle.getString(CommonDefine.PROPERTIES_STMP_KEY_NUMBER_OF_DISPLAY_ALBUM_CONTENT)))
						.getResultList();
			}

			if (syashinList != null) {
				for (TbSyashin s : syashinList) {
					DispContentDto dcont = new DispContentDto();
					dcont.id = s.getContentsId().toString();
					String[] tInfo = getTitleInfo(s.getFileName().trim());
					if (tInfo != null) {
						dcont.title = tInfo[0];
					}

					this.albumContentList.add(dcont);
				}
			}
		}
		return "albumPictureList.jsp";
	}

	/** 送付先入力画面へ遷移する **/
	@Execute(validator = false)
	public String toInputShippingAddress() {
		// 送付先情報クリア
		this.shippingAddressForm.init();
		return "/shippingAddress/?redirect=true";
	}

	/** ページング情報の設定 **/
	private void setPagingInfo(int totalResults, int number_of_content, int page) {
		// 検索結果件数
		total = totalResults;

		if (total == 0) {
			displayStartNumber = 0;
			totalPage = 0;
		} else {
			displayStartNumber = 1 + ((page - 1) * number_of_content);

			if (total % number_of_content == 0) {
				totalPage = total / number_of_content;
			} else {
				totalPage = (total / number_of_content) + 1;
			}
		}
	}

	/** コンテンツのファイル名プレフィックスを取得 **/
	private String getTitlePrefix(String title, int length) {
		String retStr = title.substring(0, title.length() - length);
		return retStr;
	}

	/** コンテンツのファイル名プレフィックスを元に同じアルバムのコンテンツ数を取得（１の場合はアルバムではない）　 **/
	private int getAlbumContentsCount(String titlePrefix, Long shinseiNo) {
		String key = titlePrefix;
		int count = 0;

		if (shinseiNo != null) {
			key = key + ":" + shinseiNo;
			if (!this.albumContentsMap.containsKey(key)) {
				count = (int) jdbcManager
						.from(TbSyashin.class)
						.where("file_name like ? and shinsei_no = ?",
								'%' + titlePrefix + '%', shinseiNo).getCount();
				this.albumContentsMap.put(key, count);
			}
		} else {
			if (!this.albumContentsMap.containsKey(key)) {
				count = (int) jdbcManager
						.from(TbSyashin.class)
						.where("file_name like ? and shinsei_no is null",
								'%' + titlePrefix + '%').getCount();
				this.albumContentsMap.put(key, count);
			}
		}

		return this.albumContentsMap.get(key);
	}

	/** 除外（アルバムから）予定のコンテンツを確定する **/
	@Execute(validator = false)
	public String defineExclude() {
		// ブラウザキャッシュを無効にする処理
		HttpServletResponse response = ResponseUtil.getResponse();
		response.setDateHeader("Expires", -1);
		response.setHeader("Pragma", "no-cache");
		response.setHeader("Cache-Control", "no-cache");

		ResponseUtil.write("true");

		return null;
	}

	/** 除外（アルバムから）予定のコンテンツを追加 **/
	@Execute(validator = false)
	public String addExcludeContent() {
		String[] array = this.pictureListForm.excludeContentIdList.split(";");

		boolean exist = false;
		for (String id : array) {
			if (id.equals(this.pictureListForm.excludeContentId)) {
				exist = true;
				break;
			}
		}
		if (!exist) {
			this.pictureListForm.excludeContentIdList += this.pictureListForm.excludeContentId
					+ ";";
		}

		// ブラウザキャッシュを無効にする処理
		HttpServletResponse response = ResponseUtil.getResponse();
		response.setDateHeader("Expires", -1);
		response.setHeader("Pragma", "no-cache");
		response.setHeader("Cache-Control", "no-cache");

		ResponseUtil.write(this.pictureListForm.excludeContentIdList);
		return null;
	}

	/** 除外（アルバムから）予定のコンテンツを削除 **/
	@Execute(validator = false)
	public String deleteExcludeContent() {
		String[] array = this.pictureListForm.excludeContentIdList.split(";");

		String tmp = "";
		for (String id : array) {
			if (!id.equals(this.pictureListForm.excludeContentId)) {
				tmp += id + ";";
			}
		}
		this.pictureListForm.excludeContentIdList = tmp;

		// ブラウザキャッシュを無効にする処理
		HttpServletResponse response = ResponseUtil.getResponse();
		response.setDateHeader("Expires", -1);
		response.setHeader("Pragma", "no-cache");
		response.setHeader("Cache-Control", "no-cache");

		ResponseUtil.write(this.pictureListForm.excludeContentIdList);
		return null;
	}

	/** 日付の妥当性チェック **/
	/** 妥当でない場合は、エラーメッセージを配列として返す **/
	private boolean isDateValid() {
		boolean isValid = true;
		ArrayList<String> array = new ArrayList<String>();

		// fromが"全て設定されいてる"または"全て設定されていない"以外の場合はエラー
		if (!(((this.pictureListForm.fromYear == -1)
				&& (this.pictureListForm.fromMonth == -1) && (this.pictureListForm.fromDay == -1)) || ((this.pictureListForm.fromYear != -1)
				&& (this.pictureListForm.fromMonth != -1) && (this.pictureListForm.fromDay != -1)))) {
			array.add(this.ERROR_DATE_FROM_NOT_ALL);
		}
		// toが"全て設定されいてる"または"全て設定されていない"以外の場合はエラー
		if (!(((this.pictureListForm.toYear == -1)
				&& (this.pictureListForm.toMonth == -1) && (this.pictureListForm.toDay == -1)) || ((this.pictureListForm.toYear != -1)
				&& (this.pictureListForm.toMonth != -1) && (this.pictureListForm.toDay != -1)))) {
			array.add(this.ERROR_DATE_TO_NOT_ALL);
		}

		// 開始と終了の逆転がないかチェックする
		if ((this.pictureListForm.fromYear != -1)
				&& (this.pictureListForm.fromMonth != -1)
				&& (this.pictureListForm.fromDay != -1)
				&& (this.pictureListForm.toYear != -1)
				&& (this.pictureListForm.toMonth != -1)
				&& (this.pictureListForm.toDay != -1)) {
			SimpleDateFormat format = new SimpleDateFormat("yyyy/MM/dd");

			try {
				Date fromDate = format.parse(Integer
						.toString(this.pictureListForm.fromYear)
						+ "/"
						+ Integer.toString(this.pictureListForm.fromMonth)
						+ "/" + Integer.toString(this.pictureListForm.fromDay));

				Date toDate = format.parse(Integer
						.toString(this.pictureListForm.toYear)
						+ "/"
						+ Integer.toString(this.pictureListForm.toMonth)
						+ "/"
						+ Integer.toString(this.pictureListForm.toDay));
				if (fromDate.compareTo(toDate) > 0) {
					array.add(this.ERROR_DATE_CONVERSE);
				}
			} catch (ParseException e) {
				LOGGER.error("ERROR", e);
				e.printStackTrace();
			}
		}

		if (!array.isEmpty()) {
			dateErrMessage = (String[]) array.toArray(new String[0]);
			isValid = false;
		}

		return isValid;
	}

	/** パンクズリスト用の文言を作成 **/
	private void setBreadCrumbList() {
		// パンクズリスト用に市町村名を取得
		TbShichouson shi = jdbcManager.from(TbShichouson.class)
				.where("shichouson_code = ?", userDto.shichousonCode)
				.getSingleResult();
		TbKen ken = jdbcManager.from(TbKen.class)
				.where("ken_code = ?", shi.getKenCode()).getSingleResult();
		this.municipality = ken.getKenName().trim()
				+ shi.getShichousonName().trim();
	}

	/** 検索条件の設定 **/
	private SearchContentCriteria setCriteria() {
		SearchContentCriteria criteria = new SearchContentCriteria();

		if (this.pictureListForm.searchCondition == 0) {
			// 日付検索
			if ((this.pictureListForm.fromYear != -1)
					&& (this.pictureListForm.fromMonth != -1)
					&& (this.pictureListForm.fromDay != -1)) {
				Calendar cal1 = Calendar.getInstance();
				cal1.set(this.pictureListForm.fromYear,
						this.pictureListForm.fromMonth - 1,
						this.pictureListForm.fromDay, 0, 0, 0);
				criteria.setPublishedFrom(cal1.getTime());
			} else {
				criteria.setPublishedFrom(null);
			}

			if ((this.pictureListForm.toYear != -1)
					&& (this.pictureListForm.toMonth != -1)
					&& (this.pictureListForm.toDay != -1)) {
				Calendar cal2 = Calendar.getInstance();
				cal2.set(this.pictureListForm.toYear,
						this.pictureListForm.toMonth - 1,
						this.pictureListForm.toDay, 23, 59, 59);
				criteria.setPublishedTo(cal2.getTime());
			} else {
				criteria.setPublishedTo(null);
			}
			criteria.setText(null);
		} else if (this.pictureListForm.searchCondition == 1) {
			// フリーワード検索
			criteria.setText(this.pictureListForm.freeText);
		} else {
			// タグ検索
			criteria.setTags(this.pictureListForm.tag);

		}

		// quanpからの移行でプレースの概念が無くなり、
		// 検索対象としては毎回return_flgがnullもしくはpublic_flgがtrueの写真を検索することで固定なので不要。
		// criteria.setPlaces(userDto.photoPlace.getId());
		return criteria;
	}

	// ////////////////////////////////////////////////////////////////////////////////
	// 以下は顔検索機能の追加コード
	// ////////////////////////////////////////////////////////////////////////////////
	/**
	 * 顔検索画面を表示します
	 * 
	 **/
	@Execute(validator = false)
	public String faceSearch() {
		LOGGER.debug("顔検索画面");
		String errorPage = "/common/error.jsp";

		if (userDto.account == null) {
			return errorPage;
		}
		String contentId = pictureListForm.facePictureContentId;
		SimpleWhere where = new SimpleWhere();
		where.eq("contentsId", contentId);
		where.eq("quanpId", userDto.account);

		if (contentId.equals(CommonDefine.NONE_CONTENTS_ID)) { // ローカルから写真をアップロードする
			pictureListForm.tbFaces = null;
			pictureListForm.iconUrl = "";
			pictureListForm.pictureFrom = 0;
			pictureListForm.imageWidth = 0;
		} else { // サーバー上の写真を利用する
			try {
				BufferedImage bufImg = null;
				TbSyashin syashin = jdbcManager.from(TbSyashin.class)
						.where(where).getSingleResult();
				double scale = 1;
				ResourceBundle bundle = ResourceBundle
						.getBundle(CommonDefine.PROPERTIES_STMP_FILE_NAME);

				if (syashin != null) {
					StorageS3Service s3 = new StorageS3Service();
					
					// ファイルキーを作成
					String key = bundle
							.getString(CommonDefine.PROPERTIES_STMP_KEY_IMG_VIEW_PREFIX)
							+ contentId
							+ bundle.getString(CommonDefine.PROPERTIES_STMP_KEY_IMG_SUFFIX);

					bufImg = ImageIO.read(s3.downloadFile(userDto.account, key));

						int imgWidth = bufImg.getWidth();
						scale = IMAGE_WIDTH_MAXSIZE / (double) imgWidth;
						if (scale < 1) {
							imgWidth = (int) (imgWidth * scale);
						} else {
							scale = 1;
						}
						pictureListForm.imageWidth = imgWidth;

					pictureListForm.tbFaces = facesearchService
							.findFacesFromPicture(contentId, scale);
					pictureListForm.iconUrl = "/imageGet/getView/" + contentId;
					pictureListForm.pictureFrom = 1;
					pictureListForm.featureFilePath = "";
				} else {
					LOGGER.debug("写真がDBに登録していない。");
					return errorPage;
				}
			} catch (FileNotFoundException e) {
				LOGGER.error(" , USER : " + userDto.account, e);
				return errorPage;
			} catch (IOException e) {
				LOGGER.error(" , USER : " + userDto.account, e);
				return errorPage;
			} catch (NumberFormatException e) {
				LOGGER.error(" , USER : " + userDto.account, e);
				return errorPage;
			}
		}

		return "faceSearch.jsp";
	}

	/**
	 * 顔写真をアップロード
	 * 
	 */
	@Execute(validator = false)
	public String uploadPicture() throws FileNotFoundException, IOException {
		Boolean exceeded = (Boolean) request
				.getAttribute(MultipartRequestHandler.ATTRIBUTE_MAX_LENGTH_EXCEEDED);
		if (exceeded != null && exceeded.booleanValue()) {
			Map<String, String> map = new HashMap<String, String>();
			map.put("error", "サイズが5MB以内の写真をご利用ください。");
			String json = JSON.encode(map);
			ResponseUtil.getResponse().setCharacterEncoding("UTF-8");
			ResponseUtil.write(json, "text/html");
			return null;
		}

		if (!isValidSuffix(pictureListForm.facePicture.getFileName())) {
			Map<String, String> map = new HashMap<String, String>();
			map.put("error", "JPEGファイルをご利用してください。");
			String json = JSON.encode(map);
			ResponseUtil.getResponse().setCharacterEncoding("UTF-8");
			ResponseUtil.write(json, "text/html");
			return null;
		}
		try {
			byte[] icoBytes = pictureListForm.facePicture.getFileData();
			BufferedImage image = ImageIO.read(new ByteArrayInputStream(
					icoBytes, 0, icoBytes.length));
			int imageWidth = image.getWidth();
			double scale = IMAGE_WIDTH_MAXSIZE / (double) imageWidth;
			if (scale < 1) {
				imageWidth = (int) (imageWidth * scale);
			}

			pictureListForm.iconUrl = "loadFacePciture?v=" + UUID.randomUUID();
			pictureListForm.imageWidth = imageWidth;
			pictureListForm.pictureFrom = 0;

			Map<String, String> map = new HashMap<String, String>();
			map.put("iconUrl", "loadFacePciture?v=" + UUID.randomUUID());
			map.put("imageWidth", "" + imageWidth);
			map.put("pictureFrom", "" + 0);
			String json = JSON.encode(map);
			ResponseUtil.getResponse().setCharacterEncoding("UTF-8");
			ResponseUtil.write(json, "text/html");
			return null;
		} catch (IOException e) {
			Map<String, String> map = new HashMap<String, String>();
			map.put("error", "写真アップロードが失敗しました。もう一度やり直してください。");
			String json = JSON.encode(map);
			ResponseUtil.getResponse().setCharacterEncoding("UTF-8");
			ResponseUtil.write(json, "text/html");
			return null;
		}
	}

	private boolean isValidSuffix(String fileName) {
		boolean result = false;
		int point = fileName.lastIndexOf(".");
		if (point != -1) {
			String suffix = fileName.substring(point + 1);
			suffix = suffix.toLowerCase();
			if (suffix.equals("jpg")) {
				result = true;
			}
		}
		return result;
	}

	/**
	 * アップロードした顔写真を表示する
	 * 
	 */
	@Execute(validator = false)
	public String loadFacePciture() throws IOException {
		if (pictureListForm.facePicture == null) {
			return null;
		}
		response.setContentType("image/jpeg");
		BufferedOutputStream bos = new BufferedOutputStream(
				response.getOutputStream());
		byte[] datas = pictureListForm.facePicture.getFileData();
		if (datas == null) {
			LOGGER.debug("Load Icon Failure: datas is null ");
			return null;
		}

		try {
			bos.write(datas);
			bos.flush();
		} catch (IOException e) {
			LOGGER.debug("IOException :", e);
			return null;
		} finally {
			try {
				bos.close();
			} catch (Throwable e) {
			}
		}
		return null;
	}
}