package jp.savethememory.action;

import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.transaction.UserTransaction;

import jp.savethememory.dto.DispContentDto;
import jp.savethememory.dto.UserDto;
import jp.savethememory.entity.TbKoukaiSyashin;
import jp.savethememory.entity.TbShinsei;
import jp.savethememory.entity.TbSyashin;
import jp.savethememory.entity.TbZip;
import jp.savethememory.form.PictureListForm;
import jp.savethememory.form.ShippingAddressForm;
import jp.savethememory.util.CommonDefine;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.seasar.extension.jdbc.JdbcManager;
import org.seasar.framework.aop.annotation.Trace;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;


@Trace
public class ShippingAddressAction {
	public JdbcManager jdbcManager;
	public String birthdayErr = null;
	public String searchBirthdayErr = null;
	public String searchMessage = null;
	//公開協力画面に表示するリスト
	public List<DispContentDto> dispPublicContentList;
	
	@Resource
	@ActionForm
	protected ShippingAddressForm shippingAddressForm;
	
	@Resource
	protected PictureListForm pictureListForm;
	
	@Resource
	protected UserDto userDto;
	
	@Resource
	protected UserTransaction userTransaction;
	
	protected Log LOGGER = LogFactory.getLog(ShippingAddressAction.class);
	private String BIRTHDAY_ERROR_MESSAGE = "生年月日が不正です。";
	
	/**
	 * 送付先情報入力画面を表示します
	 * @return
	 */
	@Execute(validator=false)
	public String index(){
		this.pictureListForm.from = 2;
		this.shippingAddressForm.selectedCount = this.pictureListForm.selectedCount;
		
		this.shippingAddressForm.hasPublicContent = false;
		//申請の中に公開写真が含まれるか判断
		String[] checkedTitlePrefixList = this.pictureListForm.selectedTitlePrefix.split(";");
		for(String titlePrefix : checkedTitlePrefixList){
			long count = 0;
			count = jdbcManager.from(TbSyashin.class)
					.where("file_name like ? and public_flg is true", '%' + titlePrefix + '%')
					.getCount();
			if(count > 0){
				this.shippingAddressForm.hasPublicContent = true;
				break;
			}
		}
		
		return "index.jsp";
	}
	
	/**
	 * 送付先情報確認画面を表示します
	 * @return
	 */
	@Execute(validator=true , input="index.jsp")
	public String confirm(){
		String ret = "confirm.jsp";
		
		//生年月日の不正チェック(アノテーションでは対応できないため)
		try {
			getBirthday(this.shippingAddressForm.birthdayYear 
					, this.shippingAddressForm.birthdayMonth 
					, this.shippingAddressForm.birthdayDay);
		} catch(ParseException e) {
			birthdayErr = BIRTHDAY_ERROR_MESSAGE;
			ret = "index.jsp";
		}
		
		return ret;
	}
	
	/**
	 * 申請処理行います。
	 * 申請写真中に集合写真が存在する場合は、公開協力確認画面を表示します。
	 * 存在しない場合は、申請完了画面を表示します。
	 * @return
	 */
	@Execute(validator=false)
	public String apply() {
		String ret = "complete.jsp";
		if(this.pictureListForm.selectedCount != 0){
			
			this.shippingAddressForm.resultCount = 0;
			this.shippingAddressForm.shinseiNo = 0;
			String[] checkedTitlePrefixList = this.pictureListForm.selectedTitlePrefix.split(";");
			
			List<TbSyashin> syashinList = new ArrayList<TbSyashin>();
			List<TbSyashin> publicSyashinList = new ArrayList<TbSyashin>();
			List<Long> excludeIdList = new ArrayList<Long>();
			
			for(String titlePrefix : checkedTitlePrefixList){
				////対象のコンテンツが"公開コンテンツではない かつ 申請されていない"　又は　"公開コンテンツである"　場合は申請可能
				//除外写真を考慮
				if(!this.pictureListForm.excludeContentIdList.isEmpty()){
					String[] idList = this.pictureListForm.excludeContentIdList.split(";");
					
					for(String id : idList){
						excludeIdList.add(Long.parseLong(id));
					}
				}
				
				//公開対応ではない写真検索
				List<TbSyashin> list = new ArrayList<TbSyashin>();
				
				list = jdbcManager.from(TbSyashin.class)
						.where("file_name like ? and shinsei_no is null and public_flg is not true", '%' + titlePrefix + '%')
						.getResultList();
				if(!list.isEmpty()){
					//除外対象には除外フラグを設定
					if(!this.pictureListForm.excludeContentIdList.isEmpty()){
						String[] idList = this.pictureListForm.excludeContentIdList.split(";");
						
						for(TbSyashin s : list){
							for(String id : idList){
								if(id.equals(s.getContentsId().toString())){
									s.setExcludeFlg(true);
								}
							}
						}
					}
					syashinList.addAll(list);
				} else if(this.shippingAddressForm.hasPublicContent){
					//公開対応の写真検索
					List<TbSyashin> pList = new ArrayList<TbSyashin>();
					pList = jdbcManager.from(TbSyashin.class)
								.where("file_name like ? and public_flg is true", '%' + titlePrefix + '%')
								.getResultList();
					if(!pList.isEmpty()){
						publicSyashinList.addAll(pList);
					}
				}
			}
			
			//1件でも申請されていないコンテンツがあれば申請情報insert（=すべてのコンテンツが申請中の場合（先に申請されていた場合）申請情報はinsertしない）
			if(!syashinList.isEmpty() || !publicSyashinList.isEmpty()){
				//申請情報insert(申請Noは自動採番)
				TbShinsei shinsei = new TbShinsei();
				setShinseiInfo(shinsei);
				
				int shinseiInsertCount = jdbcManager.insert(shinsei).execute();
				
				if(shinseiInsertCount != 0){
					//申請番号はDBのシーケンスを利用
					this.shippingAddressForm.shinseiNo = jdbcManager
																.selectBySql(Long.class,"select currval('tb_shinsei_shinsei_no_seq')")
																.getSingleResult();
					
					int[] countArray;
					//tb_syashin更新
					if(!syashinList.isEmpty()){
						for(TbSyashin syashin : syashinList){
							syashin.setShinseiNo(this.shippingAddressForm.shinseiNo);
						}
						countArray = jdbcManager.updateBatch(syashinList).execute();
						this.shippingAddressForm.resultCount = countArray.length;
					}
					
					//tb_koukai_syashin挿入
					if(!publicSyashinList.isEmpty()){
						List<TbKoukaiSyashin> insertList = new ArrayList<TbKoukaiSyashin>();
						for(TbSyashin syashin : publicSyashinList){
							TbKoukaiSyashin s = new TbKoukaiSyashin();
							s.setContentsId(syashin.getContentsId());
							//s.setQuanpUploadStatus(quanpUploadStatus)
							s.setShinseiNo(this.shippingAddressForm.shinseiNo);
							s.setFileName(syashin.getFileName());
							insertList.add(s);
						}
						countArray = jdbcManager.insertBatch(insertList).execute();
						this.shippingAddressForm.resultCount += countArray.length;
					}
				}
			}
			
			this.pictureListForm.selectedTitlePrefix = "";
			this.pictureListForm.selectedCount = 0;
			this.pictureListForm.excludeContentIdList = "";
			
			//公開写真が存在する場合は公開協力確認画面を表示.存在しない場合は完了画面を表示
			setPublicPictureList();
			if(!this.dispPublicContentList.isEmpty()){
				ret = "publicPictureList.jsp";
			} 
			
		} else {
			setPublicPictureList();
			if(!this.dispPublicContentList.isEmpty()){
				ret = "publicPictureList.jsp";
			} 
		}
		
		return ret;
	}
	
	/**
	 * 写真選択確認画面へ戻ります
	 * @return
	 */
	@Execute(validator=false)
	public String cancel(){
		return "/pictureList/toConfirmSelectedPicture?redirect=true";
	}
	
	/**
	 * 公開協力確認画面用情報を設定します。
	 */
	@Execute(validator=false)
	private void setPublicPictureList(){
		//申請の中に公開写真が存在する場合は公開写真画面を表示する
		this.shippingAddressForm.publicSyashinList = new ArrayList<TbSyashin>();
		this.dispPublicContentList = new ArrayList<DispContentDto>();
    	
    	if(this.shippingAddressForm.shinseiNo != 0){
    		//集合写真を表示対象とする
			List<TbSyashin> syashinList = jdbcManager.from(TbSyashin.class)
												.where("shinsei_no = ? and group_flg is true and exclude_flg is not true", this.shippingAddressForm.shinseiNo)
												.orderBy("fileName asc")
												.getResultList();
			
			if(syashinList != null && syashinList.size() > 0){
				for(TbSyashin syashin : syashinList){
					this.shippingAddressForm.publicSyashinList.add(syashin);
					DispContentDto dcont = new DispContentDto();
		    		dcont.id = syashin.getContentsId().toString();
		    		
		    		//ファイル名の先頭が日付の場合は削除して表示
		    		if(CommonDefine.FILE_NAME_PATTERN1.matcher(syashin.getFileName().trim()).find()
		    			|| CommonDefine.FILE_NAME_PATTERN2.matcher(syashin.getFileName().trim()).find()){
		    			dcont.title = syashin.getFileName().trim().substring(CommonDefine.FILE_NAME_SUBSTRING_DATE_LENGTH);
		    		} else {
		    			dcont.title = syashin.getFileName().trim();
		    		}
					
		    		this.dispPublicContentList.add(dcont);
				}
			}
    	}
	}
	
	/**
	 * 申請情報をセットする
	 * @param shinsei
	 */
	private void setShinseiInfo(TbShinsei shinsei){
		try{
			shinsei.setShichousonCode(userDto.shichousonCode);
			shinsei.setShinseiNameSei(this.shippingAddressForm.lastName);
			shinsei.setShinseiNameMei(this.shippingAddressForm.firstName);
			shinsei.setShinseiFuriganaSei(this.shippingAddressForm.lastNameKana);
			shinsei.setShinseiFuriganaMei(this.shippingAddressForm.firstNameKana);
			shinsei.setTelNo1(this.shippingAddressForm.telNo1);
			shinsei.setTelNo2(this.shippingAddressForm.telNo2);
			shinsei.setTelNo3(this.shippingAddressForm.telNo3);
			shinsei.setSex(this.shippingAddressForm.sex);	
			shinsei.setGenJushoJushoZipCode(this.shippingAddressForm.presentZipcode);
			shinsei.setGenJushoJushoPrefecture(this.shippingAddressForm.presentPrefecture);
			shinsei.setGenJushoJushoCity(this.shippingAddressForm.presentCity);
			shinsei.setGenJushoJushoBantiGou(this.shippingAddressForm.presentAddress);
			shinsei.setGenJushoJushoTatemono(this.shippingAddressForm.presentBuilding);
			shinsei.setJuuminhyouJushoZipCode(this.shippingAddressForm.residenceCertZipcode);
			shinsei.setJuuminhyouJushoPrefecture(this.shippingAddressForm.residenceCertPrefecture);
			shinsei.setJuuminhyouJushoCity(this.shippingAddressForm.residenceCertCity);
			shinsei.setJuuminhyouJushoBantiGou(this.shippingAddressForm.residenceCertAddress);
			shinsei.setJuuminhyouJushoTatemono(this.shippingAddressForm.residenceCertBuilding);
			//生年月日
			Date birthday = getBirthday(this.shippingAddressForm.birthdayYear , 
											this.shippingAddressForm.birthdayMonth , 
											this.shippingAddressForm.birthdayDay);
			shinsei.setBirthday(birthday);
			shinsei.setMailAddress(this.shippingAddressForm.mail);
			shinsei.setShinseishaComment(this.shippingAddressForm.message);
			shinsei.setHikiwatashiSyudan(0);
			shinsei.setShinseiStatus(0);
			shinsei.setDigitalStatus(0);
			shinsei.setShinseiDay(new Date());
			shinsei.setUpdateDay( new Timestamp(System.currentTimeMillis()));
		} catch(ParseException e) {
			//事前チェックを行っているためここにくることはないはず
			LOGGER.error(" , USER : " + userDto.account , e);
		} 
	}
	
	/**
	 * 写真公開に協力する、を選択した場合の処理を行います
	 * @return
	 */
	@Execute(validator=false)
	public String cooperateWithPublicPicture(){
		//公開写真の設定を行う
		if(!this.shippingAddressForm.publicSyashinList.isEmpty()){
    		for(TbSyashin syashin : this.shippingAddressForm.publicSyashinList){
    			syashin.setPublicFlg(true);
    		}
    		jdbcManager.updateBatch(this.shippingAddressForm.publicSyashinList).execute();
    	}
		
		return "complete.jsp";
	}
	/**
	 * 写真公開に協力する、を選択しなかった場合の処理
	 * @return
	 */
	@Execute(validator=false)
	public String notCooperateWithPublicPicture(){
		return "complete.jsp";
	}
	
	/**
	 * 写真検索画面に遷移します
	 * @return
	 */
	@Execute(validator=false)
	public String toPictureList(){
		this.shippingAddressForm.init();
		this.pictureListForm.selectedTitlePrefix = "";
		this.pictureListForm.selectedCount = 0;
		return "/pictureList/?redirect=true";
	}
	
//	/**
//	 * 写真選択確認画面に遷移します。
//	 * @return
//	 */
//	@Execute(validator=false)
//	public String toConfirmSelectedPicture(){
//		return "/pictureList/toConfirmSelectedPicture?redirect=true";
//	}
	
	
	/**
	 * 電話番号と生年月日から送付先の履歴を検索します。
	 * 該当する情報が存在する場合は、最新の情報を送付先情報としてセットします。
	 */
	@Execute(validator=true , input="index.jsp")
	public String searchShippingAddressHistory(){
		//エラーチェック
		Date birthday = null;
		try {
			birthday = getBirthday(this.shippingAddressForm.searchBirthdayYear ,
											this.shippingAddressForm.searchBirthdayMonth ,
											this.shippingAddressForm.searchBirthdayDay);
		} catch (ParseException e) {
			searchBirthdayErr = BIRTHDAY_ERROR_MESSAGE;
		}
		
		if(searchBirthdayErr == null){
			//電話番号と生年月日からtb_shinseiを検索
			List<TbShinsei> shinseiList = jdbcManager.from(TbShinsei.class).
											where("tel_no1 = ? and tel_no2 = ? and tel_no3 = ? and birthday = ?",
											this.shippingAddressForm.searchTelNo1,
											this.shippingAddressForm.searchTelNo2,
											this.shippingAddressForm.searchTelNo3,
											birthday).orderBy("shinseiNo desc").getResultList();
			if(shinseiList.isEmpty()){
				searchMessage = "該当する送付先情報はありませんでした";
			} else {
				//複数件存在する場合はメッセージを表示
				if(shinseiList.size() > 1){
					searchMessage = "該当する送付先情報が複数件存在するため、最新の情報を設定しました";
				}
				//送付先情報セット
				TbShinsei shinsei = shinseiList.get(0);
				this.shippingAddressForm.lastName = shinsei.getShinseiNameSei().trim();
				this.shippingAddressForm.firstName = shinsei.getShinseiNameMei().trim();
				this.shippingAddressForm.lastNameKana = shinsei.getShinseiFuriganaSei().trim();
				this.shippingAddressForm.firstNameKana = shinsei.getShinseiFuriganaMei().trim();
				this.shippingAddressForm.telNo1 = this.shippingAddressForm.searchTelNo1;
				this.shippingAddressForm.telNo2 = this.shippingAddressForm.searchTelNo2;
				this.shippingAddressForm.telNo3 = this.shippingAddressForm.searchTelNo3;
				this.shippingAddressForm.sex = shinsei.getSex();		
				this.shippingAddressForm.presentZipcode = shinsei.getGenJushoJushoZipCode().trim();
				this.shippingAddressForm.presentPrefecture = shinsei.getGenJushoJushoPrefecture().trim();
				this.shippingAddressForm.presentCity = shinsei.getGenJushoJushoCity().trim();
				this.shippingAddressForm.presentAddress = shinsei.getGenJushoJushoBantiGou().trim();
				this.shippingAddressForm.presentBuilding = shinsei.getGenJushoJushoTatemono().trim();
				this.shippingAddressForm.residenceCertZipcode = shinsei.getJuuminhyouJushoZipCode().trim();
				this.shippingAddressForm.residenceCertPrefecture = shinsei.getJuuminhyouJushoPrefecture().trim();
				this.shippingAddressForm.residenceCertCity = shinsei.getJuuminhyouJushoCity().trim();
				this.shippingAddressForm.residenceCertAddress = shinsei.getJuuminhyouJushoBantiGou().trim();
				this.shippingAddressForm.residenceCertBuilding = shinsei.getJuuminhyouJushoTatemono().trim();
				this.shippingAddressForm.birthdayYear = this.shippingAddressForm.searchBirthdayYear;
				this.shippingAddressForm.birthdayMonth = this.shippingAddressForm.searchBirthdayMonth;
				this.shippingAddressForm.birthdayDay = this.shippingAddressForm.searchBirthdayDay;
				this.shippingAddressForm.mail = shinsei.getMailAddress().trim();
				this.shippingAddressForm.retypeMail = shinsei.getMailAddress().trim();
			}
		}
		
		return "index.jsp";
	}
	
	/**
	 * 郵便番号から住所を検索します。
	 * 該当する情報が存在する場合は、現住所欄に設定します。
	 * @return
	 */
	@Execute(validator=true , input="index.jsp")
	public String searchPresentAddress(){
		TbZip tbZip = null;
		tbZip = jdbcManager.from(TbZip.class).
							where("zip_code = ?",this.shippingAddressForm.presentZipcode).getSingleResult();
		
		if(tbZip == null){
			searchMessage = "該当する住所はありませんでした";
			this.shippingAddressForm.presentPrefecture = null;
			this.shippingAddressForm.presentCity = null;
		} else {
			this.shippingAddressForm.presentPrefecture = tbZip.getPrefecture();
			this.shippingAddressForm.presentCity = tbZip.getCity()+tbZip.getStreet();
		}
	
		return "index.jsp";
	}
	
	/**
	 * 郵便番号から住所を検索します。
	 * 該当する情報が存在する場合は、住民票住所欄に設定します。
	 * @return
	 */
	@Execute(validator=true , input="index.jsp")
	public String searchResidenceCertAddress(){
		TbZip tbZip = null;
		tbZip = jdbcManager.from(TbZip.class).
							where("zip_code = ?",this.shippingAddressForm.residenceCertZipcode).getSingleResult();
		
		if(tbZip == null){
			searchMessage = "該当する住所はありませんでした";
		} else {
			this.shippingAddressForm.residenceCertPrefecture = tbZip.getPrefecture();
			this.shippingAddressForm.residenceCertCity = tbZip.getCity()+tbZip.getStreet();
		}
	
		return "index.jsp";
	}
	
	/**
	 * 年月日のフォーマットを行います。
	 */
	private Date getBirthday(String year , String month , String day) throws ParseException{
		Date birthday = null;
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd");
		sdf.setLenient(false);
		birthday = sdf.parse(year + "/" + month + "/" + day);
		return birthday;
	}
	
}
