package jp.savethememory.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Comparator;
import java.util.List;
import java.util.ResourceBundle;

import jp.savethememory.entity.TbFace;
import jp.savethememory.entity.TbSyashin;
import jp.savethememory.facerecognize.FaceVerifier;
import jp.savethememory.facerecognize.VerifyException;
import jp.savethememory.facerecognize.VerifyResult;
import jp.savethememory.util.CommonDefine;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.seasar.extension.jdbc.JdbcManager;
import org.seasar.extension.jdbc.where.ComplexWhere;
import org.seasar.extension.jdbc.where.SimpleWhere;
import org.seasar.framework.aop.annotation.Trace;

@Trace
public class FacesearchService {

	public JdbcManager jdbcManager;

	protected Log LOGGER = LogFactory.getLog(FacesearchService.class);

	public List<TbFace> findFacesFromPicture(String contentId_str, double scale) {
		long contentsId = Long.parseLong(contentId_str);
		SimpleWhere where = new SimpleWhere();
		where.eq("contentsId", contentsId);
		List<TbFace> faces = jdbcManager.from(TbFace.class).where(where)
				.getResultList();

		for (TbFace face : faces) {
			face.setScale(scale);
		}

		return faces;
	}

	public List<TbSyashin> findPictureListByLocalFace(byte[] datas,
			String quanpId) throws VerifyException {
		ResourceBundle bundle = ResourceBundle
				.getBundle(CommonDefine.PROPERTIES_STMP_FILE_NAME);
		String fetFilePath = bundle.getString(CommonDefine.FET_FILES_ROOT_PATH)
				+ "/" + quanpId;

		FaceVerifier verify = FaceVerifier.getInstance();
		long starttime = Calendar.getInstance().getTimeInMillis();
		LOGGER.info("[SearchTime] NeoFace Process Start(Use Local Picture) :　"
				+ starttime);
		VerifyResult[] verifyArray = verify.verifyJPEGWithMultiFace(datas,
				fetFilePath, CommonDefine.verifyScore);
		long stoptime = Calendar.getInstance().getTimeInMillis();
		LOGGER.info("[SearchTime] NeoFace Process Finish(Use Local Picture) :"
				+ stoptime);
		LOGGER.info("[SearchTime] NeoFace Process Cost Time(Use Local Picture) :"
				+ (stoptime - starttime));
		if (verifyArray == null) {
			return null;
		}

		List<TbSyashin> verifyResults = createSortContentResults(verifyArray);

		return verifyResults;
	}

	public List<TbSyashin> findPictureListByServerFace(String fetName,
			String quanpId) throws VerifyException {
		ResourceBundle bundle = ResourceBundle
				.getBundle(CommonDefine.PROPERTIES_STMP_FILE_NAME);
		String fetFilePath = bundle.getString(CommonDefine.FET_FILES_ROOT_PATH)
				+ "/" + quanpId;

		FaceVerifier verify = FaceVerifier.getInstance();
		long starttime = Calendar.getInstance().getTimeInMillis();
		LOGGER.info("[SearchTime] NeoFace Process Start(Use Server Face) :"
				+ starttime);
		VerifyResult[] verifyArray = verify.verifyFeatureWithMultiFace(
				fetName.trim(), fetFilePath, CommonDefine.verifyScore);
		long stoptime = Calendar.getInstance().getTimeInMillis();
		LOGGER.info("[SearchTime] NeoFace Process Finish(Use Server Face) :"
				+ stoptime);
		LOGGER.info("[SearchTime] NeoFace Process Cost Time(Use Server Face) :　"
				+ (stoptime - starttime));

		if (verifyArray == null || verifyArray.length == 0) {
			return null;
		}

		List<TbSyashin> verifyResults = createSortContentResults(verifyArray);

		return verifyResults;
	}

	private List<TbSyashin> createSortContentResults(VerifyResult[] verifyArray) {
		Arrays.sort(verifyArray, new Comparator<VerifyResult>() {
			public int compare(VerifyResult v1, VerifyResult v2) {
				return (v2.getScore() - v1.getScore() > 0 ? 1 : -1);
			}
		});

		List<TbSyashin> pictureVerifyResults = new ArrayList<TbSyashin>();
		List<String> resultsContentIds = new ArrayList<String>();
		long starttime = Calendar.getInstance().getTimeInMillis();
		LOGGER.info("[SearchTime] Get Contents List Start: " + starttime);
		for (VerifyResult verifyResult : verifyArray) {
			String contentId = String.valueOf(verifyResult.getContentId());
			TbSyashin localContent = selectContentFromDb(contentId);
			if (localContent != null && !resultsContentIds.contains(contentId)) {
				pictureVerifyResults.add(localContent);
				resultsContentIds.add(contentId);
			}
			if (pictureVerifyResults.size() >= CommonDefine.VERIFY_RESULT_CONTENT_MAX_COUNT) {
				break;
			}
		}
		long stoptime = Calendar.getInstance().getTimeInMillis();
		LOGGER.info("[SearchTime] Get Contents List Finish: " + stoptime);
		LOGGER.info("[SearchTime] Get Contents List Cost Time: "
				+ (stoptime - starttime));

		return pictureVerifyResults;
	}

	private TbSyashin selectContentFromDb(String contentsId) {
		// DBから、返却の実施の有無や公開写真かどうかを確認する。
		TbSyashin content = null;

		SimpleWhere contentsIdWhere = new SimpleWhere().eq("contentsId",
				contentsId);
		
		ComplexWhere returnFlgOrPublicPlgWhere = NormalSearchService.createSearchableTbSyashinComplexWhere();

		content = jdbcManager.from(TbSyashin.class)
				.where(contentsIdWhere, returnFlgOrPublicPlgWhere)
				.getSingleResult();

		return content;
	}

	public List<String> receiveHasFaceContentIds(List<TbSyashin> contents) {
		if (contents == null || contents.size() <= 0) {
			return null;
		}
		String conditions = "'" + contents.get(0).getContentsId() + "'";
		for (int i = 1; i < contents.size(); i++) {
			conditions += ("," + "'" + contents.get(i).getContentsId() + "'");
		}
		return jdbcManager.selectBySql(
				String.class,
				"select distinct contents_id from tb_face where contents_id in ("
						+ conditions + ")").getResultList();
	}

}
