/*  Copyright© Ricoh IT Solutions Co.,Ltd.
 * All Rights Reserved.
 */
package jp.savethememory.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import jp.savethememory.entity.TbSyashin;
import jp.savethememory.entity.TbSyashinList;

import org.seasar.extension.jdbc.JdbcManager;
import org.seasar.extension.jdbc.where.ComplexWhere;
import org.seasar.extension.jdbc.where.SimpleWhere;

import jp.savethememory.entity.SearchContentCriteria;

/**
 *
 * @version 0.1
 * @since 2013/01/24
 */
public class NormalSearchService {

	public JdbcManager jdbcManager;
	private static final int SEARCH_TYPE_DATE = 0;
	private static final int SEARCH_TYPE_FREE_WORD = 1;
	private static final String BASE_SQL = "select distinct "
			+ "tb_syashin.contents_id, directory_name, file_name, shinsei_no, return_flg, quanp_upload_status, published_day, group_flg, public_flg, exclude_flg, quanp_id, summary "
			+ "from tb_syashin left outer join tb_tag on tb_tag.contents_id = tb_syashin.contents_id "
			+ "where "
			+ "quanp_id = ? and (return_flg is null or public_flg = true) ";

	/**
	 * フリーワード検索時に利用される条件
	 */
	private static final String WHERE_FREE_WORD = "and ("
			+ "tb_tag.name ilike '%' || ? || '%' "
			+ "or  tb_syashin.file_name ilike '%' || ? || '%' "
			+ "or  tb_syashin.summary ilike '%' || ? || '%') ";

	public TbSyashinList searchTbSyashinList(String quanpId,
			int searchCondition, SearchContentCriteria criteria,
			int contentsCountPerPage, int page) {

		// 検索ヒット数を取得
		int allResultCount = countTbSyashin(quanpId, searchCondition, criteria);

		// 検索ヒット数をセット。
		TbSyashinList targetSearchResultList = new TbSyashinList();
		targetSearchResultList.setTotalResults(allResultCount);

		// 表示するコンテンツをQuanpから取得
		List<TbSyashin> targetTbSYashinList = searchTargetTbSyashinList(
				quanpId, searchCondition, criteria, contentsCountPerPage, page);

		// 表示対象のリストをセット
		targetSearchResultList.setTbSyashinList(targetTbSYashinList);

		// 総ヒット数と、1ページあたりのコンテンツ数や
		// 何ページのオフセットがあるかや、
		// 次のページでいくつの写真を表示するかを比較して、
		// 更なる検索結果があるかどうかを確認してセット。
		targetSearchResultList.setHasNextSearchResult(targetSearchResultList
				.getTotalResults() > targetTbSYashinList.size()
				+ contentsCountPerPage * (page - 1));

		return targetSearchResultList;

	}

	/**
	 * DBからTbSyashinを検索. 検索対象は、quanpIdに紐づけられた、return_flgがfalseな写真。
	 *
	 * @return TbSyashinList 検索結果
	 */
	private List<TbSyashin> searchTargetTbSyashinList(String quanpId,
			int searchCondition, SearchContentCriteria criteria,
			int contentsCountPerPage, int page) {

		List<TbSyashin> resultList = null;

		// ORマッパーの引数となるmapを作成
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("freeWord", criteria.getText());
		param.put("tagName", criteria.getTags());
		param.put("quanpId", quanpId);

		if (searchCondition == SEARCH_TYPE_DATE) {// 日付検索

			SimpleWhere quanpIdWhere = new SimpleWhere().eq("quanpId", quanpId)
					.ge("publishedDay", criteria.getPublishedFrom())
					.le("publishedDay", criteria.getPublishedTo());

			// リターンフラグがnull（＝未返却）、もしくはpublicフラグがtrue（＝公開写真）
			ComplexWhere returnFlgOrPublicPlgWhere = createSearchableTbSyashinComplexWhere();

			resultList = jdbcManager.from(TbSyashin.class)
					.where(quanpIdWhere, returnFlgOrPublicPlgWhere)
					.limit(contentsCountPerPage)
					.offset((page - 1) * contentsCountPerPage)
					.orderBy("fileName asc").getResultList();

		} else if (searchCondition == SEARCH_TYPE_FREE_WORD) {// フリーワード検索

			List<String> paramList = new ArrayList<String>();

			String sql = makeFreeWordSqlAndLoadParams(criteria.getText(),
					quanpId, paramList);

			sql = sql + "order by file_name asc";

			Object[] params = paramList.toArray(new String[paramList.size()]);

			resultList = jdbcManager.selectBySql(TbSyashin.class, sql, params)
					.limit(contentsCountPerPage)
					.offset((page - 1) * contentsCountPerPage).getResultList();

		} else {// タグ検索

			resultList = jdbcManager
					.selectBySqlFile(TbSyashin.class,
							"sql/tb_syashin/searchTbSyashinByTag.sql", param)
					.limit(contentsCountPerPage)
					.offset((page - 1) * contentsCountPerPage).getResultList();
		}

		return resultList;
	}

	/**
	 * 検索のヒット数をリターンする.
	 *
	 * @return 検索のヒット数
	 */
	private int countTbSyashin(String quanpId, int searchCondition,
			SearchContentCriteria criteria) {

		// ORマッパーの引数となるmapを作成
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("freeWord", criteria.getText());
		param.put("tagName", criteria.getTags());
		param.put("quanpId", quanpId);

		// 戻り値
		int allResultCount = 0;

		if (searchCondition == SEARCH_TYPE_DATE) {// 日付検索

			SimpleWhere quanpIdWhere = new SimpleWhere().eq("quanpId", quanpId)
					.ge("publishedDay", criteria.getPublishedFrom())
					.le("publishedDay", criteria.getPublishedTo());

			ComplexWhere returnFlgOrPublicPlgWhere = createSearchableTbSyashinComplexWhere();

			// 総件数
			allResultCount = (int) jdbcManager.from(TbSyashin.class)
					.where(quanpIdWhere, returnFlgOrPublicPlgWhere).getCount();

		} else if (searchCondition == SEARCH_TYPE_FREE_WORD) {// フリーワード検索
			List<String> paramList = new ArrayList<String>();

			String sql = makeFreeWordSqlAndLoadParams(criteria.getText(),
					quanpId, paramList);

			Object[] params = paramList.toArray(new String[paramList.size()]);

			allResultCount = (int) jdbcManager.getCountBySql(sql, params);

		} else {// タグ検索
			allResultCount = (int) jdbcManager.getCountBySqlFile(
					"sql/tb_syashin/countTbSyashinByTag.sql", param);
		}

		return allResultCount;
	}

	/**
	 * DBからTbSyashinを検索. 検索対象は、quanpIdに紐づけられた、return_flgがfalseな写真。
	 *
	 * @param contentsId
	 *            コンテンツID
	 * @return TbSyashinList 検索結果
	 */
	public TbSyashin searchTbSyashin(int contentsId) {

		TbSyashin resultTbSyashin = null;

		SimpleWhere whereContentsId = new SimpleWhere().eq("contentsId",
				contentsId);

		ComplexWhere returnFlgOrPublicPlgWhere = createSearchableTbSyashinComplexWhere();

		resultTbSyashin = jdbcManager.from(TbSyashin.class)
				.where(whereContentsId, returnFlgOrPublicPlgWhere)
				.getSingleResult();

		return resultTbSyashin;
	}

	/**
	 * リターンフラグがnull（＝未返却）、もしくはpublicフラグがtrue（＝公開写真）なものが検索可能
	 *
	 * @return
	 */
	protected static ComplexWhere createSearchableTbSyashinComplexWhere() {
		return new ComplexWhere().isNull("returnFlg", true).or()
				.eq("publicFlg", true);
	}

	/**
	 * フリーワード検索のためのPreparedStatmentの基となる文字列を返します、
	 * 第2引数のリストの末尾にPreparedStatementを実行するための引数となる要素を加えます.
	 *
	 * @param rawFreeWord
	 *            ユーザが入力したフリーワード
	 * @param whereStringList
	 *            引数を加えたいリスト
	 * @return PreparedStatementを実行するうえで引数となる配列（例：{"stmpakita", "A", "A", "A",
	 *         "B", "B", "B"}）
	 */
	private static String addFreeWordWhereAndParams(String rawFreeWord,
			List<String> whereStringList) {

		String sql = "";

		// フリーワードは+や　を複数検索ワードの区切りに使用できるので
		// それらを半角スペースに置換
		String freeWord = rawFreeWord.replace("+", " ").replace("　", " ");

		String[] freeWordArray = freeWord.split(" ");
		for (String word : freeWordArray) {
			// スペースが連続していた場合はsplitで空文字列が得られるのでそれは無視
			if (!"".equals(word)) {
				// 空文字列でないなら条件を追加
				sql = sql + WHERE_FREE_WORD;

				// 3回addするのは複数単語での検索のうち1つの単語につき3つある"?"のすべてに
				// 値を入れるから
				whereStringList.add(word);
				whereStringList.add(word);
				whereStringList.add(word);
			}
		}

		return sql;

	}

	/**
	 * フリーワード検索のためのSQLを返し、第3引数のリストには、SQLをPreparedStatementで実行するための引数を格納します.
	 *
	 * @param freeWord
	 *            検索時に入力されたフリーワード。
	 * @param quanpId
	 *            検索対象のID
	 * @param paramList
	 *            空のリスト。
	 * @return フリーワード検索のためのSQL
	 */
	private static String makeFreeWordSqlAndLoadParams(String freeWord,
			String quanpId, List<String> paramList) {
		// 共通部分のSQL
		String sql = BASE_SQL;
		paramList.add(quanpId);

		// ""での囲みを処理するために"でsplit
		String[] splitedFreeWords = freeWord.split("\"");

		for (int i = 0; i < splitedFreeWords.length; i++) {
			if (!"".equals(splitedFreeWords[i])) {
				// 空文字列は検索単語とみなさない
				// 空文字列ではない場合

				if (i % 2 == 0) {
					// "でsplitして、0はじまりで偶数個目の要素は""で囲まれていない
					// したがってスペースでの単語区切りが適用される。
					sql = sql
							+ addFreeWordWhereAndParams(splitedFreeWords[i],
									paramList);
				} else {
					// 0はじまりで奇数個目の要素は""で囲まれている。
					// したがってスペースを単語区切りとみなさずにそのまま文字列の一部として検索する。
					sql = sql + WHERE_FREE_WORD;

					// 3回addするのは複数単語での検索のうち1つの単語につき3つある"?"のすべてに
					// 値を入れるから
					paramList.add(splitedFreeWords[i]);
					paramList.add(splitedFreeWords[i]);
					paramList.add(splitedFreeWords[i]);
				}
			}
		}
		return sql;
	}

}
