package jp.savethememory.service;

import jp.savethememory.entity.TbShichouson;
import jp.savethememory.entity.UserInfo;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.seasar.extension.jdbc.JdbcManager;
import org.seasar.extension.jdbc.where.SimpleWhere;
import org.seasar.framework.aop.annotation.Trace;
import org.seasar.struts.annotation.Execute;

@Trace
public class UserInfoService {

	public JdbcManager jdbcManager;

	protected Log LOGGER = LogFactory.getLog(UserInfoService.class);

	/**
	 * 指定されたアカウントがSTMの利用自治体のアカウントであることを確認する 自治体コードも特定したい。
	 * さらに、quanpで認証し、認証がOKであれば、UserInfoを生成して返却する。 該当しない場合はUserInfoはnullで返す。
	 * */
	@Execute
	public UserInfo getUserInfo(String account, String password) {

		UserInfo userInfo = isExistDB(account, password);
		return userInfo;
	}

	/**
	 * 指定されたアカウントがSTMの利用自治体のアカウントであること（DBに存在するか）を確認する。
	 * 該当するアカウントが存在しない場合はUserInfoはnullで返す。
	 * */
	@Execute
	protected UserInfo isExistDB(String account, String password) {
		UserInfo userInfo = null;

		// ローカルのデータベースサーバーにアクセスし、認証処理を行なう
		TbShichouson result = jdbcManager
				.from(TbShichouson.class)
				.where(new SimpleWhere().eq("quanpId", account).eq(
						"quanpPassword", password)).getSingleResult();

		if (result != null) {
			userInfo = new UserInfo();
			userInfo.setAccount(account);
			userInfo.setShichousonCode(result.getShichousonCode());
			userInfo.setShichousonName(result.getShichousonName());
		}
		if (userInfo != null) {
			LOGGER.debug(account + " is Exsits in STMDB.");
		}
		return userInfo;
	}
}